@@ -68,6 +68,11 @@ struct VTypedef;
 struct VXformInfo;
 struct PageMapIterator;
 
+struct KDataBuffer;
+struct KNamelist;
+struct BSTree;
+struct VColumn;
+struct VCtxId;
 
 /*--------------------------------------------------------------------------
  * VDBManager
@@ -188,6 +193,18 @@ VDB_EXTERN bool CC VDatabaseIsCSRA ( struct VDatabase const *self );
 VDB_EXTERN rc_t CC VDatabaseGetAccession(const struct VDatabase * self,
     const struct String ** acc, const struct String ** path);
 
+/* VDatabaseMemberType
+ * Characterize a database member
+ */
+enum
+{
+    dbmUnknown,
+    dbmDatabase,
+    dbmTable,
+    dbmViewAlias
+};
+VDB_EXTERN rc_t VDatabaseMemberType(const struct VDatabase * self, const char * name, uint32_t * type);
+
 /*--------------------------------------------------------------------------
  * VTable
  */
@@ -323,6 +340,17 @@ VDB_EXTERN rc_t CC VCursorLinkedCursorSet(const struct VCursor *cself,const char
 VDB_EXTERN uint64_t CC VCursorSetCacheCapacity(struct VCursor *self,uint64_t capacity);
 VDB_EXTERN uint64_t CC VCursorGetCacheCapacity(const struct VCursor *self);
 
+/* ListReadableColumns
+ *  performs an insert of '*' to cursor
+ *  attempts to resolve all read rules
+ *  records all SColumns that successfully resolved
+ *  populates BTree with VColumnRef objects
+ */
+rc_t VCursorListReadableColumns ( struct VCursor *self, struct BSTree *columns );
+
+/* GetColumn
+*/
+VDB_EXTERN struct VColumn * VCursorGetColumn(struct VCursor * self, const struct VCtxId * ctx);
 
 /*--------------------------------------------------------------------------
  * VCursorParams
