@@ -40,6 +40,7 @@
 #include <klib/defs.h>
 #include <klib/rc.h>
 #include <klib/vector.h>
+#include <klib/text.h>
 #include <kproc/queue.h>
 #include <kproc/thread.hpp>
 #include <kproc/timeout.h>
@@ -445,14 +446,14 @@ rc_t process_alignment( SAMExtractor * state, const char * qname,
     Alignment * align = (Alignment *)pool_alloc( sizeof( *align ) );
 
     align->qname = qname;
-    align->flags = fast_strtoi64( flag );
+    align->flags = (uint16_t)fast_strtoi64( flag );
     align->rname = rname;
     align->pos   = ipos;
-    align->mapq  = fast_strtoi64( mapq );
+    align->mapq  = (uint8_t)fast_strtoi64( mapq );
     align->cigar = cigar;
     align->rnext = rnext;
-    align->pnext = fast_strtoi64( pnext );
-    align->tlen  = fast_strtoi64( tlen );
+    align->pnext = (uint32_t)fast_strtoi64( pnext );
+    align->tlen  = (uint32_t)fast_strtoi64( tlen );
     align->read  = seq;
     align->qual  = qual;
     VectorAppend( &state->alignments, NULL, align );
@@ -540,7 +541,7 @@ LIB_EXPORT rc_t CC SAMExtractorMake( SAMExtractor ** state, const KFile * fin,
     pool_init();
 
     s->infile  = fin;
-    fname_desc = strdup( fname->addr );
+    fname_desc = string_dup_measure( fname->addr, NULL );
 //    s->fname = fname_desc;
 
     VectorInit( &s->headers, 0, 0 );
