@@ -20,7 +20,7 @@
  *
  *  Please cite the author in any work or product based on this material.
  *
- * =============================================================================
+ * ============================================================================$
  *
  */
 
@@ -197,40 +197,69 @@ static bool ReaderColsIsReadCompressed(const ReaderCols *self,
 
 /******************************************************************************/
 
+/* When oNreads != NULL: get number of reads from READ_LEN (NEVER USED);
+   read column otherwise. */
 static
 uint32_t _VCursorReadArray(const VCursor *self,
     int64_t row_id,
     uint32_t col,
     void **buffer,
     uint8_t elem_size,
-    uint8_t nReads,
-    const char *name)
+    uint32_t nreads,
+    const char *name,
+    uint32_t *oNreads)
 {
     rc_t rc = 0;
     uint32_t row_len = 0;
 
-    assert(buffer && elem_size && nReads && name);
+    assert(name);
 
-    if (*buffer == NULL) {
-        *buffer = calloc(nReads, elem_size);
-        if (*buffer == NULL) {
-            return eVdbBlastMemErr;
+    if (oNreads != NULL) {
+        uint32_t status = eVdbBlastNoErr;
+        uint32_t elem_bits = 0, elem_off = 0, elem_cnt = 0;
+        const void *base = NULL;
+        assert(oNreads == NULL);/*NEVER USED*/
+        rc = VCursorCellDataDirect(self, row_id, col,
+            &elem_bits, &base, &elem_off, &elem_cnt);
+        if (rc != 0) {
+            status = eVdbBlastErr;
+            PLOGMSG(klogInfo, (klogInfo,
+                "$(f): Cannot '$(name)' CellDataDirect",
+                "f=%s,name=%s", __func__, name));
         }
+        else if (elem_off != 0 || elem_bits != 32) {
+            status = eVdbBlastErr;
+            PLOGERR(klogInt, (klogInt, rc,
+                "Bad VCursorCellDataDirect(READ_LEN) result: "
+                "boff=$(elem_off), elem_bits=$(elem_bits)",
+                "elem_off=%u, elem_bits=%u", elem_off, elem_bits));
+        }
+        else
+            *oNreads = elem_cnt;
+        return status;
     }
+    else {
+        assert(buffer && elem_size && nreads);
 
-    rc = VCursorReadDirect(self,
-        row_id, col, 8, *buffer, nReads * elem_size * 8, &row_len);
-    if (rc != 0) {
-        PLOGERR(klogInt, (klogInt, rc,
-            "Error in VCursorReadDirect($(name))", "name=%s", name));
+        if (*buffer == NULL) {
+            *buffer = calloc(nreads, elem_size);
+            if (*buffer == NULL)
+                return eVdbBlastMemErr;
+        }
+
+        rc = VCursorReadDirect(self,
+            row_id, col, 8, *buffer, nreads * elem_size * 8, &row_len);
+        if (rc != 0)
+            PLOGERR(klogInt, (klogInt, rc,
+                "Error in VCursorReadDirect($(name))", "name=%s", name));
+
+        /* TODO: needs to be verified what row_len is expected
+            if (row_len != 1) return eVdbBlastErr; */
+
+        S
+
+        return rc == 0 ? eVdbBlastNoErr : eVdbBlastErr;
     }
-
-/* TODO: needs to be verified what row_len is expected
-    if (row_len != 1) return eVdbBlastErr; */
-
-    S
-
-    return rc == 0 ? eVdbBlastNoErr : eVdbBlastErr;
 }
 
 static
@@ -323,18 +352,18 @@ static uint32_t _VCursorReadReaderCols(const VCursor *self,
     rc_t rc = 0;
     uint32_t row_len = ~0;
     int64_t row_id = 0;
-    uint8_t nReads = 0;
+    uint32_t nreads = 0;
 
     assert(desc && cols && desc->run && empty);
 
     * empty = false;
 
     row_id = desc->spot;
-    nReads = desc->run->rd.nReads;
+    nreads = desc->run->rd.nReads;
     assert(desc->nReads);
-    nReads = desc->nReads;
+    nreads = desc->nReads;
 
-    if (cols->nReadsAllocated != 0 && cols->nReadsAllocated < nReads) {
+    if (cols->nReadsAllocated != 0 && cols->nReadsAllocated < nreads) {
         /* LOG */
 
         /* TODO: find a better way/place to realloc cols data buffers */
@@ -349,37 +378,39 @@ static uint32_t _VCursorReadReaderCols(const VCursor *self,
     }
 
     status = _VCursorReadArray(self, row_id, cols->col_READ_LEN,
-        (void **)&cols->read_len, sizeof *cols->read_len, nReads,
-        "READ_LEN");
+        (void **)&cols->read_len, sizeof *cols->read_len, nreads,
+        "READ_LEN", NULL);
     if (status != eVdbBlastNoErr)
         return status;
-    if ( nReads == 1 && cols->read_len[0] == 0 ) {
+    if ( nreads == 1 && cols->read_len[0] == 0 ) {
         * empty = true;
         return status;
     }
 
     status = _VCursorReadArray(self, row_id, cols->col_READ_FILTER,
-        (void **)&cols->read_filter, sizeof *cols->read_filter, nReads,
-        "READ_FILTER");
+        (void **)&cols->read_filter, sizeof *cols->read_filter, nreads,
+        "READ_FILTER", NULL);
     if (status != eVdbBlastNoErr)
     {   return status; }
 
     status = _VCursorReadArray(self, row_id, cols->col_READ_TYPE,
-        (void **)&cols->read_type, sizeof *cols->read_type, nReads,
-        "READ_TYPE");
+        (void **)&cols->read_type, sizeof *cols->read_type, nreads,
+        "READ_TYPE", NULL);
     if (status != eVdbBlastNoErr)
         return status;
 
     if (cols->col_PRIMARY_ALIGNMENT_ID != 0) {
         status = _VCursorReadArray(self, row_id, cols->col_PRIMARY_ALIGNMENT_ID,
             (void **)&cols->primary_alignment_id,
-            sizeof *cols->primary_alignment_id, nReads, "PRIMARY_ALIGNMENT_ID");
+            sizeof *cols->primary_alignment_id, nreads, "PRIMARY_ALIGNMENT_ID",
+            NULL);
         if (status != eVdbBlastNoErr) {
             return status;
         }
     }
 
-    cols->nReadsAllocated = nReads;
+    if (cols->nReadsAllocated < nreads)
+        cols->nReadsAllocated = nreads;
 
     rc = VCursorReadDirect(self, row_id, cols->col_TRIM_LEN,
         8 * sizeof cols->TRIM_LEN, &cols->TRIM_LEN, sizeof cols->TRIM_LEN,
@@ -420,7 +451,7 @@ bool _ReadDescNextRead(ReadDesc *self, VdbBlastStatus *status)
     uint32_t read = 0;
     int i = 0;
     const RunDesc *rd = NULL;
-    uint8_t nReads = 1;
+    uint32_t nreads = 1;
 
     assert(self && self->run && status);
 
@@ -449,9 +480,9 @@ bool _ReadDescNextRead(ReadDesc *self, VdbBlastStatus *status)
         }
 
         if (self->tableId == VDB_READ_UNALIGNED)
-            nReads = rd->nReads;
+            nreads = rd->nReads;
 
-        for (i = self->read + 1; i <= nReads; ++i) {
+        for (i = self->read + 1; i <= nreads; ++i) {
             if (rd->readType[i - 1] & SRA_READ_TYPE_BIOLOGICAL) {
                 S
                 read = i;
@@ -465,7 +496,7 @@ bool _ReadDescNextRead(ReadDesc *self, VdbBlastStatus *status)
                 return false;
             }
 
-            for (i = 1; i <= nReads; ++i) {
+            for (i = 1; i <= nreads; ++i) {
                 if (rd->readType[i - 1] & SRA_READ_TYPE_BIOLOGICAL) {
                     S
                     read = i;
@@ -652,7 +683,8 @@ static
 uint32_t _Reader2naCalcReadReaderColsParams(const ReadDesc *desc,
     const ReaderCols *cols,
     uint32_t *start,
-    uint32_t min_read_length)
+    uint32_t min_read_length,
+    bool ignoreReadFilter)
 {
     uint32_t i = 1;
     uint32_t to_read = 0;
@@ -682,7 +714,9 @@ uint32_t _Reader2naCalcReadReaderColsParams(const ReadDesc *desc,
             __func__, desc->run->path, desc->spot, desc->read, desc->read_id));
         return 0;
     }
-    else if (cols->read_filter[desc->read - 1] != READ_FILTER_PASS) {
+    else if (!ignoreReadFilter
+        && cols->read_filter[desc->read - 1] != READ_FILTER_PASS)
+    {
         S
         DBGMSG(DBG_BLAST, DBG_FLAG(DBG_BLAST_BLAST),
             ("%s: %s:%d:%d(%d): READ_FILTER != READ_FILTER_PASS\n",
@@ -755,7 +789,8 @@ bool _Reader2naNextData(Reader2na *self,
     const VBlob *blob,
     uint32_t *status,
     Packed2naRead *out,
-    uint32_t min_read_length)
+    uint32_t min_read_length,
+    bool ignoreReadFilter)
 {
     uint32_t start = 0;
     uint32_t to_read = 0;
@@ -784,7 +819,7 @@ bool _Reader2naNextData(Reader2na *self,
     }
 
     to_read = _Reader2naCalcReadReaderColsParams(&self->desc, &self->cols,
-        &start, min_read_length);
+        &start, min_read_length, ignoreReadFilter);
     if (to_read == 0) {
         return true;
     }
@@ -847,7 +882,8 @@ uint32_t _Reader2naData(Reader2na *self,
     VdbBlastStatus *status,
     Packed2naRead *buffer,
     uint32_t buffer_length,
-    uint32_t min_read_length)
+    uint32_t min_read_length,
+    bool ignoreReadFilter)
 {
     ReadDesc *desc = NULL;
     uint32_t n = 0;
@@ -872,8 +908,8 @@ uint32_t _Reader2naData(Reader2na *self,
 
     for (n = 0; n < buffer_length; ) {
         Packed2naRead *p = buffer + n;
-        bool ignorable
-            = _Reader2naNextData(self, data->blob, status, p, min_read_length);
+        bool ignorable = _Reader2naNextData(
+            self, data->blob, status, p, min_read_length, ignoreReadFilter);
         if (*status == eVdbBlastErr) {
             if (ignorable) {
                 /* special case */
@@ -920,7 +956,8 @@ uint64_t _Reader2naRead(Reader2na *self,
     size_t *starting_base,
     uint8_t *buffer,
     size_t buffer_size,
-    uint32_t min_read_length)
+    uint32_t min_read_length,
+    bool ignoreReadFilter)
 {
     uint32_t num_read = 0;
     uint32_t to_read = 0;
@@ -962,7 +999,7 @@ uint64_t _Reader2naRead(Reader2na *self,
         }
 
         to_read = _Reader2naCalcReadReaderColsParams(&self->desc, &self->cols,
-            &start, min_read_length);
+            &start, min_read_length, ignoreReadFilter);
         if (to_read <= self->starting_base) {
             S
             DBGMSG(DBG_BLAST, DBG_FLAG(DBG_BLAST_BLAST), (
@@ -1199,7 +1236,8 @@ uint32_t _Core2naDataSeq(Core2na *self,
     const RunSet *runs,
     VdbBlastStatus *status,
     Packed2naRead *buffer,
-    uint32_t buffer_length)
+    uint32_t buffer_length,
+    bool ignoreReadFilter)
 {
     uint32_t num_read = 0;
 
@@ -1231,7 +1269,7 @@ uint32_t _Core2naDataSeq(Core2na *self,
         }
 
         num_read = _Reader2naData(&self->reader, data, status,
-            buffer, buffer_length, self->min_read_length);
+            buffer, buffer_length, self->min_read_length, ignoreReadFilter);
     }
 
     if (*status == eVdbBlastNoErr) {
@@ -1250,12 +1288,14 @@ uint32_t _Core2naData(Core2na *self,
     const RunSet *runs,
     VdbBlastStatus *status,
     Packed2naRead *buffer,
-    uint32_t buffer_length)
+    uint32_t buffer_length,
+    bool ignoreReadFilter)
 {
     assert(self);
 
     if (self->reader.mode != VDB_READ_REFERENCE) {
-        return _Core2naDataSeq(self, data, runs, status, buffer, buffer_length);
+        return _Core2naDataSeq(self, data, runs, status, buffer, buffer_length,
+            ignoreReadFilter);
     }
     else {
         return _Core2naDataRef(self, data, status, buffer, buffer_length);
@@ -1269,7 +1309,8 @@ uint64_t _Core2naReadSeq(Core2na *self,
     uint64_t *read_id,
     size_t *starting_base,
     uint8_t *buffer,
-    size_t buffer_size)
+    size_t buffer_size,
+    bool ignoreReadFilter)
 {
     uint64_t num_read = 0;
 
@@ -1298,8 +1339,8 @@ uint64_t _Core2naReadSeq(Core2na *self,
             return 0;
         }
 
-        num_read = _Reader2naRead(&self->reader, status,
-            read_id, starting_base, buffer, buffer_size, self->min_read_length);
+        num_read = _Reader2naRead(&self->reader, status, read_id, starting_base,
+            buffer, buffer_size, self->min_read_length, ignoreReadFilter);
         S
     }
 
@@ -1308,17 +1349,15 @@ uint64_t _Core2naReadSeq(Core2na *self,
 
 uint64_t _Core2naRead(Core2na *self, const RunSet *runs,
     uint32_t *status, uint64_t *read_id, size_t *starting_base,
-    uint8_t *buffer, size_t buffer_size)
+    uint8_t *buffer, size_t buffer_size, bool ignoreReadFilter)
 {
     assert(self);
 
-    if (self->reader.mode != VDB_READ_REFERENCE) {
-        return _Core2naReadSeq(self, runs, status,
-            read_id, starting_base, buffer, buffer_size);
-    }
-    else {
+    if (self->reader.mode == VDB_READ_REFERENCE)
         return _Core2naReadRef(self, status, read_id, buffer, buffer_size);
-    }
+    else
+        return _Core2naReadSeq(self, runs, status,
+            read_id, starting_base, buffer, buffer_size, ignoreReadFilter);
 }
 
 void _Core4naFini(Core4na *self) {
@@ -1335,7 +1374,8 @@ void _Core4naFini(Core4na *self) {
 
 static size_t _Core4naReadSeq(Core4na *self, const RunSet *runs,
     uint32_t *status, uint64_t read_id, size_t starting_base,
-    uint8_t *buffer, size_t buffer_length)
+    uint8_t *buffer, size_t buffer_length,
+    bool ignoreReadFilter)
 {
     uint32_t num_read = 0;
     ReadDesc *desc = NULL;
@@ -1373,7 +1413,8 @@ static size_t _Core4naReadSeq(Core4na *self, const RunSet *runs,
                 if (!empty) {
                     assert(self->cols.read_len && self->cols.read_filter);
                     to_read = _Reader2naCalcReadReaderColsParams(&self->desc,
-                        &self->cols, &start, self->min_read_length);
+                        &self->cols, &start, self->min_read_length,
+                        ignoreReadFilter);
                 }
                 if (to_read == 0) {
                     /* When _Reader2naCalcReadReaderColsParams returns 0
@@ -1419,13 +1460,14 @@ static size_t _Core4naReadSeq(Core4na *self, const RunSet *runs,
 
 static size_t _Core4naRead(Core4na *self, const RunSet *runs,
     uint32_t *status, uint64_t read_id, size_t starting_base,
-    uint8_t *buffer, size_t buffer_length)
+    uint8_t *buffer, size_t buffer_length,
+    bool ignoreReadFilter)
 {
     assert(self);
 
     if (self->mode != VDB_READ_REFERENCE) {
         return _Core4naReadSeq(self, runs, status,
-            read_id, starting_base, buffer, buffer_length);
+            read_id, starting_base, buffer, buffer_length, ignoreReadFilter);
     }
     else {
         return _Core4naReadRef(self, runs, status,
@@ -1434,7 +1476,7 @@ static size_t _Core4naRead(Core4na *self, const RunSet *runs,
 }
 
 static const uint8_t* _Core4naDataSeq(Core4na *self, const RunSet *runs,
-    uint32_t *status, uint64_t read_id, size_t *length)
+    uint32_t *status, uint64_t read_id, size_t *length, bool ignoreReadFilter)
 {
     ReadDesc *desc = NULL;
     assert(self && runs && status && length);
@@ -1476,8 +1518,8 @@ static const uint8_t* _Core4naDataSeq(Core4na *self, const RunSet *runs,
             assert(self->cols.read_len && self->cols.read_filter
                 && desc->run->path);
 
-            if (self->cols.read_filter[desc->read - 1]
-                != READ_FILTER_PASS)
+            if (!ignoreReadFilter
+                && self->cols.read_filter[desc->read - 1] != READ_FILTER_PASS)
             {   /* FILTERed reads are not returned by 2na reader:
                    4na readed should not ask for them */
                 *status = eVdbBlastInvalidId;
@@ -1538,8 +1580,8 @@ static const uint8_t* _Core4naDataSeq(Core4na *self, const RunSet *runs,
                             else {
                                 uint32_t start = 0;
                                 to_read = _Reader2naCalcReadReaderColsParams(
-                                    &self->desc, &self->cols,
-                                    &start, self->min_read_length);
+                                    &self->desc, &self->cols, &start,
+                                    self->min_read_length, ignoreReadFilter);
                                 base += boff + start;
                                 if (row_len >= start) {
                                     row_len -= start;
@@ -1584,12 +1626,13 @@ static const uint8_t* _Core4naDataSeq(Core4na *self, const RunSet *runs,
 }
 
 static const uint8_t* _Core4naData(Core4na *self, const RunSet *runs,
-    uint32_t *status, uint64_t read_id, size_t *length)
+    uint32_t *status, uint64_t read_id, size_t *length, bool ignoreReadFilter)
 {
     assert(self);
 
     if (self->mode != VDB_READ_REFERENCE) {
-        return _Core4naDataSeq(self, runs, status, read_id, length);
+        return _Core4naDataSeq(self, runs, status, read_id, length,
+            ignoreReadFilter);
     }
     else {
         return _Core4naDataRef(self, runs, status, read_id, length);
@@ -1861,8 +1904,8 @@ uint32_t CC VdbBlast2naReaderData(VdbBlast2naReader *self,
         fflush(stderr);
     }
     if (rc == 0) {
-        n = _Core2naData((Core2na*)core2na, &self->data,
-            &self->set->runs, status, buffer, buffer_length);
+        n = _Core2naData((Core2na*)core2na, &self->data, &self->set->runs,
+            status, buffer, buffer_length, self->set->ignoreReadFilter);
         S
         if (n > 0 && verbose)
         {   _Packed2naReadPrint(buffer, self->data.blob); }
@@ -1941,6 +1984,18 @@ VdbBlast4naReader* CC VdbBlastRunSetMake4naReader(const VdbBlastRunSet *self,
     return VdbBlastRunSetMake4naReaderExt(self, status, VDB_READ_UNALIGNED);
 }
 
+LIB_EXPORT
+VdbBlastStatus CC VdbBlastRunSetIgnoreReadFilter(VdbBlastRunSet* self,
+    bool ignore)
+{
+    if (self == NULL) {
+        STSMSG(1, ("VdbBlastRunSetIgnoreReadFilter(NULL)"));
+        return eVdbBlastErr;
+    }
+    self->ignoreReadFilter = ignore;
+    return eVdbBlastNoErr;
+}
+
 LIB_EXPORT
 VdbBlast4naReader* CC VdbBlast4naReaderAddRef(VdbBlast4naReader *self)
 {
@@ -2010,8 +2065,8 @@ size_t CC VdbBlast4naReaderRead(const VdbBlast4naReader *self,
         LOGERR(klogInt, rc, "Error in KLockAcquire");
     }
     else {
-        n = _Core4naRead(c, &self->set->runs, status,
-            read_id, starting_base, buffer, buffer_length);
+        n = _Core4naRead(c, &self->set->runs, status, read_id, starting_base,
+            buffer, buffer_length, self->set->ignoreReadFilter);
         rc = KLockUnlock(c->mutex);
         if (rc != 0) {
             LOGERR(klogInt, rc, "Error in KLockUnlock");
@@ -2068,7 +2123,8 @@ const uint8_t* CC VdbBlast4naReaderData(const VdbBlast4naReader *self,
         LOGERR(klogInt, rc, "Error in KLockAcquire");
     }
     else {
-        d = _Core4naData(c, &self->set->runs, status, read_id, length);
+        d = _Core4naData(c, &self->set->runs, status, read_id, length,
+            self->set->ignoreReadFilter);
         rc = KLockUnlock(c->mutex);
         if (rc != 0) {
             LOGERR(klogInt, rc, "Error in KLockUnlock");
@@ -2156,8 +2212,8 @@ void CC VdbBlastStdaaReaderRelease(VdbBlastStdaaReader *self)
     {   return; }
 
     STSMSG(1, ("Deleting VdbBlastStdaaReader"));
-    free(self);
     memset(self, 0, sizeof *self);
+    free(self);
 }
 
 LIB_EXPORT
