@@ -38,6 +38,7 @@ struct GCP;
 #include <klib/data-buffer.h>
 #include <klib/debug.h> /* DBGMSG */
 #include <klib/json.h>
+#include <klib/log.h> /* PLOGMSG */
 #include <klib/printf.h> /* string_printf */
 #include <klib/rc.h>
 #include <klib/status.h>
@@ -72,7 +73,7 @@ struct GCP;
 #define PATH_MAX 4096
 #endif
 
-static rc_t PopulateCredentials(GCP * self);
+static rc_t PopulateCredentials(GCP * self, KConfig * kfg);
 
 /* Destroy
 */
@@ -83,7 +84,6 @@ rc_t CC GCPDestroy(GCP * self)
     free(self->private_key_id);
     free(self->client_email);
     free(self->project_id);
-    free(self->access_token);
     free(self->jwt);
     return CloudWhack(&self->dad);
 }
@@ -116,7 +116,7 @@ static rc_t readCE(GCP const *const self, size_t size, char location[])
     DBGMSG(DBG_VFS, DBG_FLAG(DBG_VFS_CE),
         ("Reading GCP location from provider\n"));
     return KNSManager_Read(self->dad.kns, location, size,
-                         identityUrl, "Metadata-Flavor", "Google");
+                         identityUrl, HttpMethod_Get, "Metadata-Flavor", "Google");
 }
 
 /* MakeComputeEnvironmentToken
@@ -168,7 +168,7 @@ static rc_t GCPGetLocation(const GCP * self, const String ** location) {
     assert(self);
 
     rc = KNSManager_Read(self->dad.kns, b, sizeof b,
-        zoneUrl, "Metadata-Flavor", "Google");
+        zoneUrl, HttpMethod_Get, "Metadata-Flavor", "Google");
 
     if (rc == 0)
         slash = string_rchr(b, sizeof b, '/');
@@ -275,7 +275,8 @@ const String ** output)
         ret = mbedtls_pk_parse_key(&pk,
             (unsigned char *)key_PEM,
             string_measure(key_PEM, NULL) + 1,
-            NULL, 0);
+            NULL, 0,
+            mbedtls_ctr_drbg_random, &ctr_drbg);
         if (ret == 0)
         {
             /* 2. generate the checksum */
@@ -294,16 +295,15 @@ const String ** output)
                 String * out = NULL;
                 mbedtls_rsa_context * ctx = mbedtls_pk_rsa(pk);
 
-                out = malloc(sizeof(String) + ctx->len);
+                out = malloc(sizeof(String) + (uint32_t)ctx->MBEDTLS_PRIVATE(len));
                 if (out != NULL)
                 {
-                    StringInit( out, (char*)out + sizeof(String), ctx->len, (uint32_t)ctx->len );
+                    StringInit( out, (char*)out + sizeof(String), ctx->MBEDTLS_PRIVATE(len), (uint32_t)ctx->MBEDTLS_PRIVATE(len) );
 
                     ret = mbedtls_rsa_rsassa_pkcs1_v15_sign(
                         ctx,
                         mbedtls_ctr_drbg_random,
                         (void *)& ctr_drbg,
-                        MBEDTLS_RSA_PRIVATE,
                         md_type,
                         (unsigned int)dsize,
                         checksum,
@@ -315,9 +315,6 @@ const String ** output)
                         /* 4. verify the signature */
                         ret = mbedtls_rsa_rsassa_pkcs1_v15_verify(
                             mbedtls_pk_rsa(pk),
-                            NULL,
-                            NULL,
-                            MBEDTLS_RSA_PUBLIC,
                             md_type,
                             (unsigned int)dsize,
                             checksum,
@@ -523,6 +520,53 @@ GetJsonNumMember(const KJsonObject *obj, const char * name, int64_t * value)
     return KJsonGetNumber(member, value);
 }
 
+rc_t
+ParseAccessToken( const char * jsonResponse, char ** token, KTime_t * expiration )
+{
+    /* 5. Response:
+    {
+    "access_token" : "1/8xbJqaOZXSUZbHLl5EOtu1pxz3fmmetKx9W8CV4t79M",
+    "token_type" : "Bearer",
+    "expires_in" : 3600
+    }
+    */
+    KJsonValue * root;
+    char error[1024];
+    rc_t rc = KJsonValueMake(&root, jsonResponse, error, sizeof(error));
+    if (rc == 0)
+    {
+        const KJsonObject *obj = KJsonValueToObject(root);
+        if (rc == 0)
+        {
+            const char * value;
+            rc = GetJsonStringMember(obj, "access_token", &value);
+            if (rc == 0)
+            {
+                *token = string_dup(value, string_measure(value, NULL));
+                if (*token == NULL)
+                {
+                    rc = RC(rcNS, rcMgr, rcAllocating, rcMemory, rcExhausted);
+                }
+            }
+            if (rc == 0)
+            {
+                int64_t expires;
+                rc = GetJsonNumMember(obj, "expires_in", &expires);
+                if (rc == 0)
+                {
+                    *expiration = KTimeStamp() + expires;
+                }
+            }
+        }
+        KJsonValueWhack(root);
+    }
+    if (rc != 0)
+    {
+        rc = RC(rcCloud, rcUri, rcInitializing, rcFormat, rcUnexpected);
+    }
+    return rc;
+}
+
 static
 rc_t GetAccessToken(const GCP * self, const char * jwt, struct KStream * opt_conn, char ** token, KTime_t * expiration)
 {
@@ -580,7 +624,7 @@ rc_t GetAccessToken(const GCP * self, const char * jwt, struct KStream * opt_con
             rc = KClientHttpResultGetInputStream(rslt, &s);
             if (rc == 0)
             {
-                rc = KStreamRead(s, jsonResponse, sizeof(jsonResponse), &num_read);
+                rc = KStreamReadAll(s, jsonResponse, sizeof(jsonResponse), &num_read);
                 if (rc == 0)
                 {
                     if (num_read == sizeof(jsonResponse))
@@ -622,47 +666,7 @@ rc_t GetAccessToken(const GCP * self, const char * jwt, struct KStream * opt_con
 
     if (rc == 0)
     {
-        /* 5. Response:
-        {
-        "access_token" : "1/8xbJqaOZXSUZbHLl5EOtu1pxz3fmmetKx9W8CV4t79M",
-        "token_type" : "Bearer",
-        "expires_in" : 3600
-        }
-        */
-        KJsonValue * root;
-        char error[1024];
-        rc = KJsonValueMake(&root, jsonResponse, error, sizeof(error));
-        if (rc == 0)
-        {
-            const KJsonObject *obj = KJsonValueToObject(root);
-            if (rc == 0)
-            {
-                const char * value;
-                rc = GetJsonStringMember(obj, "access_token", &value);
-                if (rc == 0)
-                {
-                    *token = string_dup(value, string_measure(value, NULL));
-                    if (*token == NULL)
-                    {
-                        rc = RC(rcNS, rcMgr, rcAllocating, rcMemory, rcExhausted);
-                    }
-                }
-                if (rc == 0)
-                {
-                    int64_t expires;
-                    rc = GetJsonNumMember(obj, "expires_in", &expires);
-                    if (rc == 0)
-                    {
-                        *expiration = KTimeStamp() + expires;
-                    }
-                }
-            }
-        }
-        if (rc != 0)
-        {
-            rc = RC(rcCloud, rcUri, rcInitializing, rcFormat, rcUnexpected);
-        }
-        KJsonValueWhack(root);
+        rc = ParseAccessToken( jsonResponse, token, expiration );
     }
     return rc;
 }
@@ -684,11 +688,11 @@ rc_t CC GCPAddUserPaysCredentials(const GCP * cself, KClientHttpRequest * req, c
     {
         bool new_token = false;
         /* see if cached access_token has to be generated/refreshed */
-        if ( self->access_token == NULL ||
-             self->access_token_expiration < KTimeStamp() + 60 ) /* expires in less than a minute */
+        if ( self->dad.access_token == NULL ||
+             self->dad.access_token_expiration < KTimeStamp() + 60 ) /* expires in less than a minute */
         {
-            free(self->access_token);
-            self->access_token = NULL;
+            free(self->dad.access_token);
+            self->dad.access_token = NULL;
 
             if (self->jwt == NULL)
             {   /* first time here, create the JWT and hold on to it */
@@ -696,7 +700,7 @@ rc_t CC GCPAddUserPaysCredentials(const GCP * cself, KClientHttpRequest * req, c
             }
             if (rc == 0)
             {
-                rc = GetAccessToken(self, self->jwt, self->dad.conn, &self->access_token, &self->access_token_expiration);
+                rc = GetAccessToken(self, self->jwt, self->dad.conn, &self->dad.access_token, &self->dad.access_token_expiration);
             }
             new_token = true;
         }
@@ -717,7 +721,7 @@ rc_t CC GCPAddUserPaysCredentials(const GCP * cself, KClientHttpRequest * req, c
 
             if ( rc == 0 && new_token )
             {
-                rc = KClientHttpRequestAddHeader(req, "Authorization", "Bearer %s", self->access_token);
+                rc = KClientHttpRequestAddHeader(req, "Authorization", "Bearer %s", self->dad.access_token);
             }
 
             /* Add alt=media&userProject=<project_id> to the URL if not already there */
@@ -792,7 +796,8 @@ LIB_EXPORT rc_t CC CloudMgrMakeGCP(const CloudMgr * self, GCP ** p_gcp)
             self, user_agrees_to_pay, user_agrees_to_reveal_instance_identity );
         if ( rc == 0 )
         {
-            rc = PopulateCredentials(gcp);
+            if (user_agrees_to_pay)
+                rc = PopulateCredentials(gcp, (KConfig*)self->kfg);
             if (rc == 0)
             {
                 *p_gcp = gcp;
@@ -939,13 +944,13 @@ bool CloudMgrWithinGCP(const CloudMgr * self)
 }
 
 static
-rc_t PopulateCredentials(GCP * self)
+rc_t PopulateCredentials(GCP * self, KConfig * aKfg)
 {
     rc_t rc = 0;
 
     char buffer[PATH_MAX] = "";
 
-    char *jsonCredentials = NULL;
+    char *jsonCredentials = NULL, *jsonCredentialsOrig = NULL;;
 
     const char *pathToJsonFile = getenv("GOOGLE_APPLICATION_CREDENTIALS");
 
@@ -953,24 +958,33 @@ rc_t PopulateCredentials(GCP * self)
 
     if (pathToJsonFile == NULL || *pathToJsonFile == 0)
     {
-        KConfig * cfg = NULL;
-        rc = KConfigMake(&cfg, NULL);
+        KConfig * cfg = aKfg;
+        if (cfg == NULL)
+            rc = KConfigMake(&cfg, NULL);
 
         if (rc == 0)
             rc = KConfig_Get_Gcp_Credential_File(
                 cfg, buffer, sizeof buffer, NULL);
 
-        if (rc == 0)
+        if (rc == 0) {
             pathToJsonFile = buffer;
+            PLOGMSG ( klogInfo, ( klogInfo, "Got "
+                "GCP credential file '$(F)' from configuration",
+                "F=%s", pathToJsonFile ) );
+        }
         else
             rc = 0;
 
-        {
+        if (aKfg == NULL) {
             rc_t r2 = KConfigRelease(cfg);
             if (rc == 0 && r2 != 0)
                 rc = r2;
         }
     }
+    else
+        PLOGMSG ( klogInfo, ( klogInfo, "Got "
+            "GOOGLE_APPLICATION_CREDENTIALS file '$(F)' from environment",
+            "F=%s", pathToJsonFile ) );
 
     if (pathToJsonFile != NULL && *pathToJsonFile != 0)
     {   /* read the credentials file */
@@ -991,7 +1005,8 @@ rc_t PopulateCredentials(GCP * self)
 
         if (rc == 0)
         {
-            jsonCredentials = (char *)calloc(json_size + 1, 1);
+            jsonCredentialsOrig =
+                jsonCredentials = (char *)calloc(json_size + 1, 1);
             if (jsonCredentials == NULL)
             {
                 rc = RC(rcNS, rcMgr, rcAllocating, rcMemory, rcExhausted);
@@ -1009,6 +1024,15 @@ rc_t PopulateCredentials(GCP * self)
         if (rc == 0)
         {   /*extract the credentials */
             KJsonValue *root = NULL;
+
+            if (json_size > 3 &&
+                jsonCredentials[0] == '\xEF' &&
+                jsonCredentials[1] == '\xBB' &&
+                jsonCredentials[2] == '\xBF')
+            {   /* skip BOM */
+                jsonCredentials += 3;
+            }
+
             rc = KJsonValueMake(&root, jsonCredentials, NULL, 0);
             if (rc == 0)
             {
@@ -1070,7 +1094,7 @@ rc_t PopulateCredentials(GCP * self)
             }
         }
 
-        free(jsonCredentials);
+        free(jsonCredentialsOrig);
     }
 
     return rc;
