@@ -25,15 +25,19 @@
 */
 
 #include <kdb/extern.h>
-#include "windex-priv.h"
-#include "kdbfmt-priv.h"
+
+#include "windex.h"
+#include "kdbfmt.h"
+
 #include <klib/ptrie.h>
 #include <klib/text.h>
+#include <klib/rc.h>
+
 #include <kfs/directory.h>
 #include <kfs/file.h>
 #include <kfs/md5.h>
 #include <kfs/mmap.h>
-#include <klib/rc.h>
+
 #include <os-native.h>
 #include <sysalloc.h>
 
@@ -53,95 +57,7 @@
 #endif
 
 /*--------------------------------------------------------------------------
- * KPTrieIndex_v1
- *  persisted keymap
- */
-
-/* KPTrieIndexInit
- *  opens and initializes persisted keymap structure
- */
-rc_t KPTrieIndexInit_v1 ( KPTrieIndex_v1 *self, const KMMap *mm, bool byteswap )
-{
-    size_t size;
-    rc_t rc = KMMapSize ( mm, & size );
-    if ( rc == 0 )
-    {
-        const KDBHdr *hdr;
-        rc = KMMapAddrRead ( mm, ( const void** ) & hdr );
-        if ( rc == 0 )
-        {
-            /* try to create the pttree */
-            rc = PTrieMakeOrig ( & self -> key2id,
-                hdr + 1, size -= sizeof * hdr, byteswap );
-            if ( rc == 0 )
-            {
-                size_t ptsize = PTrieSize ( self -> key2id );
-                if ( ptsize <= size )
-                {
-                    /* just in case */
-                    self -> mm = NULL;
-
-                    /* record for projection */
-                    self -> byteswap = byteswap;
-
-                    /* it could be stored without projection */
-                    if ( ptsize == size )
-                    {
-                        self -> id2node = NULL;
-                        self -> first = self -> last = 0;
-                        return 0;
-                    }
-                            
-                    /* assume this is projection index */
-                    self -> id2node = ( void* )
-                        ( ( char* ) ( hdr + 1 ) + ptsize );
-                    size -= ptsize;
-                            
-                    /* it must have at least 4 bytes
-                       and be 4 byte aligned */
-                    if ( size >= sizeof ( uint32_t ) && ( size & 3 ) == 0 )
-                    {
-                        /* first entry is starting key
-                           remaining entries are node ids */
-                        self -> first = * self -> id2node ++;
-                        size -= sizeof self -> id2node [ 0 ];
-                        if ( size == 0 )
-                        {
-                            /* forget if empty */
-                            self -> id2node = NULL;
-                            self -> first = self -> last = 0;
-                            return 0;
-                        }
-                        /* remaining entries */
-                        self -> last = self -> first + ( size >> 2 ) - 1;
-                        return 0;
-                    }
-                }
-
-                PTrieWhack ( self -> key2id );
-                self -> key2id = NULL;
-
-                rc = RC ( rcDB, rcIndex, rcConstructing, rcTrie, rcCorrupt );
-            }
-        }
-    }
-
-    return rc;
-}
-
-/* KPTrieIndexWhack_v1
- *  closes down keymap
- */
-void KPTrieIndexWhack_v1 ( KPTrieIndex_v1 *self )
-{
-    PTrieWhack ( self -> key2id );
-    KMMapRelease ( self -> mm );
-    memset ( self, 0, sizeof * self );
-}
-
-
-/*--------------------------------------------------------------------------
- * KTrieIdxNode_v1
+ * KWTrieIdxNode_v1
  */
 
 static
@@ -160,17 +76,17 @@ void CC KTrieIdxNodeUnlink_v1 ( TNode *n, void *data )
 static
 void CC KTrieIdxNodeCaptureID_v1 ( TNode *n, void *data )
 {
-    KTrieIndex_v1 *self = data;
-    KTrieIdxNode_v1 *node = ( KTrieIdxNode_v1* ) n;
+    KWTrieIndex_v1 *self = data;
+    KWTrieIdxNode_v1 *node = ( KWTrieIdxNode_v1* ) n;
     self -> id2node [ node -> id - self -> first ] = node;
 }
 
 
 /*--------------------------------------------------------------------------
- * KTrieIndex_v1
+ * KWTrieIndex_v1
  */
 
-/* KTrieIndexWrite_v1
+/* KWTrieIndexWrite_v1
  */
 typedef struct PersistTrieData PersistTrieData;
 struct PersistTrieData
@@ -188,7 +104,7 @@ struct PersistTrieData
 };
 
 static
-rc_t CC KTrieIndexWrite_v1 ( void *param,
+rc_t CC KWTrieIndexWrite_v1 ( void *param,
     const void *buffer, size_t size, size_t *num_writ )
 {
     PersistTrieData *pb = param;
@@ -236,13 +152,13 @@ rc_t CC KTrieIndexWrite_v1 ( void *param,
     return 0;
 }
 
-/* KTrieIndexAux_v1
+/* KWTrieIndexAux_v1
  */
 static
-rc_t CC KTrieIndexAux_v1 ( void *param, const void *node, size_t *num_writ,
+rc_t CC KWTrieIndexAux_v1 ( void *param, const void *node, size_t *num_writ,
     PTWriteFunc write, void *write_param )
 {
-    const KTrieIdxNode_v1 *n = node;
+    const KWTrieIdxNode_v1 *n = node;
 
     if ( write != NULL )
     {
@@ -259,11 +175,11 @@ rc_t CC KTrieIndexAux_v1 ( void *param, const void *node, size_t *num_writ,
     return 0;
 }
 
-/* KTrieIndexPersist_v1
+/* KWTrieIndexPersist_v1
  *  write keymap to indicated location
  */
 static
-rc_t KTrieIndexPersistTrie_v1 ( const KTrieIndex_v1 *self, PersistTrieData *pb )
+rc_t KWTrieIndexPersistTrie_v1 ( const KWTrieIndex_v1 *self, PersistTrieData *pb )
 {
     rc_t rc;
     KDBHdr *hdr;
@@ -278,7 +194,7 @@ rc_t KTrieIndexPersistTrie_v1 ( const KTrieIndex_v1 *self, PersistTrieData *pb )
        using tree-internal key storage,
        capture the image size in pb */
     rc = TriePersist ( & self -> key2id, & pb -> ptt_size,
-        false, KTrieIndexWrite_v1, pb, KTrieIndexAux_v1, pb );
+        false, KWTrieIndexWrite_v1, pb, KWTrieIndexAux_v1, pb );
     if ( rc == 0 && pb -> marker != 0 )
     {
         size_t num_writ;
@@ -303,11 +219,11 @@ struct PersistReverseData
 };
 
 static
-void CC KTrieIndexRecordNodeId_v1 ( TNode *node, void *data )
+void CC KWTrieIndexRecordNodeId_v1 ( TNode *node, void *data )
 {
     PTNode pn;
     PersistReverseData *pb = data;
-    const KTrieIdxNode_v1 *n = ( const KTrieIdxNode_v1* ) node;
+    const KWTrieIdxNode_v1 *n = ( const KWTrieIdxNode_v1* ) node;
 
     /* lookup up the persisted node within image */
     uint32_t id = PTrieFind ( pb -> tt, & n -> n . key, & pn, NULL, NULL );
@@ -322,7 +238,7 @@ void CC KTrieIndexRecordNodeId_v1 ( TNode *node, void *data )
 }
 
 static
-rc_t KTrieIndexPersistProj_v1 ( const KTrieIndex_v1 *self, PersistTrieData *pb )
+rc_t KWTrieIndexPersistProj_v1 ( const KWTrieIndex_v1 *self, PersistTrieData *pb )
 {
 #if 0
     rc_t rc;
@@ -363,7 +279,7 @@ rc_t KTrieIndexPersistProj_v1 ( const KTrieIndex_v1 *self, PersistTrieData *pb )
                 pb2 . count = pb2 . notfound = 0;
 
                 /* record all id to node mappings */
-                TrieForEach ( & self -> key2id, KTrieIndexRecordNodeId_v1, & pb2 );
+                TrieForEach ( & self -> key2id, KWTrieIndexRecordNodeId_v1, & pb2 );
 
                 /* check for having written ids */
                 assert ( pb2 . count == PTrieCount ( pb2 . tt ) );
@@ -444,7 +360,7 @@ rc_t KTrieIndexPersistProj_v1 ( const KTrieIndex_v1 *self, PersistTrieData *pb )
             pb2 . count = pb2 . notfound = 0;
 
             /* record all id to node mappings */
-            TrieForEach ( & self -> key2id, KTrieIndexRecordNodeId_v1, & pb2 );
+            TrieForEach ( & self -> key2id, KWTrieIndexRecordNodeId_v1, & pb2 );
 
             /* check for having written ids */
             assert ( pb2 . count == PTrieCount ( pb2 . tt ) );
@@ -476,7 +392,7 @@ rc_t KTrieIndexPersistProj_v1 ( const KTrieIndex_v1 *self, PersistTrieData *pb )
 #endif
 }
 
-rc_t KTrieIndexPersist_v1 ( const KTrieIndex_v1 *self,
+rc_t KWTrieIndexPersist_v1 ( const KWTrieIndex_v1 *self,
     bool proj, KDirectory *dir, const char *path, bool use_md5 )
 {
     rc_t rc;
@@ -511,14 +427,14 @@ rc_t KTrieIndexPersist_v1 ( const KTrieIndex_v1 *self,
 		else
 		{
 		    tmplen = snprintf ( md5path, sizeof md5path, "%s.md5", path );
-		    
+
 		    if ( tmplen >= sizeof ( md5path ) ) /* can't be == or no NUL */
 		    {
 			rc = RC ( rcDB, rcIndex, rcPersisting, rcName, rcExcessive );
 		    }
 		    else
 		    {
-			rc = KDirectoryCreateFile ( dir, &kf, true, 0664, 
+			rc = KDirectoryCreateFile ( dir, &kf, true, 0664,
                                          kcmInit, "%s", tmpmd5name );
 			if ( rc == 0 )
 			{
@@ -552,28 +468,28 @@ rc_t KTrieIndexPersist_v1 ( const KTrieIndex_v1 *self,
                 pb . first = ~ 0;
                 pb . last = 0;
 
-                rc = KTrieIndexPersistTrie_v1 ( self, & pb );
+                rc = KWTrieIndexPersistTrie_v1 ( self, & pb );
                 if ( rc == 0 )
                 {
                     if ( proj )
-                        rc = KTrieIndexPersistProj_v1 ( self, & pb );
+                        rc = KWTrieIndexPersistProj_v1 ( self, & pb );
                 }
-                
+
                 KFileRelease ( pb . f );
                 pb . f = NULL;
             }
         }
-        
+
         free ( pb . buffer );
         pb . buffer = NULL;
-        
+
         if ( rc == 0 )
         {
-            rc = KDirectoryRename ( dir, false, tmpname, path );                        
+            rc = KDirectoryRename ( dir, false, tmpname, path );
             if ( rc == 0 )
             {
 		if ( use_md5 )
-		    rc = KDirectoryRename ( dir, false, tmpmd5name, md5path );                        
+		    rc = KDirectoryRename ( dir, false, tmpmd5name, md5path );
 		if ( rc == 0 )
 		{
 		    /* done */
@@ -581,26 +497,26 @@ rc_t KTrieIndexPersist_v1 ( const KTrieIndex_v1 *self,
 		}
             }
         }
-        
+
         /* whack temporary file */
         KDirectoryRemove ( dir, false, "%s", tmpname );
 	if ( use_md5 )
 	    KDirectoryRemove ( dir, false, "%s", tmpmd5name );
     }
-    
+
     return rc;
 }
 
 
 /* whack whack */
-void KTrieIndexWhack_v1 ( KTrieIndex_v1 *self )
+void KWTrieIndexWhack_v1 ( KWTrieIndex_v1 *self )
 {
     KPTrieIndexWhack_v1 ( & self -> pt );
     TrieWhack ( & self -> key2id, KTrieIdxNodeWhack_v1, NULL );
 }
 
 /* initialize an index from file - can be NULL */
-rc_t KTrieIndexOpen_v1 ( KTrieIndex_v1 *self, const KMMap *mm, bool byteswap )
+rc_t KWTrieIndexOpen_v1 ( KWTrieIndex_v1 *self, const KMMap *mm, bool byteswap )
 {
     rc_t rc;
 
@@ -625,25 +541,25 @@ rc_t KTrieIndexOpen_v1 ( KTrieIndex_v1 *self, const KMMap *mm, bool byteswap )
         }
     }
 
-    KTrieIndexWhack_v1 ( self );
+    KWTrieIndexWhack_v1 ( self );
     return rc;
 }
 
-/* KTrieIndexPopulate_v1
+/* KWTrieIndexPopulate_v1
  */
-typedef struct KTrieIndexPopulateData_v1 KTrieIndexPopulateData_v1;
-struct KTrieIndexPopulateData_v1
+typedef struct KWTrieIndexPopulateData_v1 KWTrieIndexPopulateData_v1;
+struct KWTrieIndexPopulateData_v1
 {
-    KTrieIndex_v1 *idx;
+    KWTrieIndex_v1 *idx;
     uint32_t id;
     rc_t rc;
 };
 
 static
-bool CC KTrieIndexPopulate_v1 ( PTNode *n, void *data )
+bool CC KWTrieIndexPopulate_v1 ( PTNode *n, void *data )
 {
     const String *key;
-    KTrieIndexPopulateData_v1 *pb = data;
+    KWTrieIndexPopulateData_v1 *pb = data;
 
     /* capture node id */
     uint32_t id;
@@ -660,12 +576,12 @@ bool CC KTrieIndexPopulate_v1 ( PTNode *n, void *data )
     pb -> rc = PTNodeMakeKey ( n, & key );
     if ( pb -> rc == 0 )
     {
-        KTrieIdxNode_v1 *node;
+        KWTrieIdxNode_v1 *node;
         pb -> rc = TNodeMake ( ( TNode** ) & node,
             sizeof * node + key -> size );
         if ( pb -> rc == 0 )
         {
-            KTrieIndex_v1 *self = pb -> idx;
+            KWTrieIndex_v1 *self = pb -> idx;
 
             StringInit ( & node -> n . key, node -> key, key -> size, key -> len );
             node -> id = id;
@@ -676,7 +592,7 @@ bool CC KTrieIndexPopulate_v1 ( PTNode *n, void *data )
             if ( pb -> rc == 0 )
             {
                 free ( ( void* ) key );
-    
+
                 /* if copying projection index */
                 if ( self -> id2node != NULL )
                 {
@@ -702,16 +618,16 @@ bool CC KTrieIndexPopulate_v1 ( PTNode *n, void *data )
     return true;
 }
 
-/* KTrieIndexAttach_v1
+/* KWTrieIndexAttach_v1
  *  attach a keymap to an existing table
  *
  *  "pkm" [ IN ] - a persisted keymap
  */
 static
-rc_t KTrieIndexAttach_v1 ( KTrieIndex_v1 *self, bool proj, uint32_t id )
+rc_t KWTrieIndexAttach_v1 ( KWTrieIndex_v1 *self, bool proj, uint32_t id )
 {
     uint32_t proj_len;
-    KTrieIndexPopulateData_v1 pb;
+    KWTrieIndexPopulateData_v1 pb;
 
 #if LIMIT_INSERTS
     proj_len = self -> pt . last - self -> pt . first + 1;
@@ -756,7 +672,7 @@ rc_t KTrieIndexAttach_v1 ( KTrieIndex_v1 *self, bool proj, uint32_t id )
     pb . idx = self;
     pb . id = id;
     pb . rc = 0;
-    PTrieDoUntil ( self -> pt . key2id, KTrieIndexPopulate_v1, & pb );
+    PTrieDoUntil ( self -> pt . key2id, KWTrieIndexPopulate_v1, & pb );
 
     /* if successful but needing to add projection index */
     if ( pb . rc == 0 && proj && self -> id2node == NULL )
@@ -790,9 +706,9 @@ rc_t KTrieIndexAttach_v1 ( KTrieIndex_v1 *self, bool proj, uint32_t id )
 
 /* insert string into trie, mapping to 32 bit id */
 static
-rc_t KTrieIndexExpandId2Node_v1 ( KTrieIndex_v1 *self, uint32_t range )
+rc_t KWTrieIndexExpandId2Node_v1 ( KWTrieIndex_v1 *self, uint32_t range )
 {
-    KTrieIdxNode_v1 **id2node;
+    KWTrieIdxNode_v1 **id2node;
     range = ( range + 4095 ) & - 4096;
     id2node = realloc ( self -> id2node, range * sizeof id2node [ 0 ] );
     if ( id2node == NULL )
@@ -811,13 +727,13 @@ rc_t KTrieIndexExpandId2Node_v1 ( KTrieIndex_v1 *self, uint32_t range )
     return 0;
 }
 
-rc_t KTrieIndexInsert_v1 ( KTrieIndex_v1 *self,
+rc_t KWTrieIndexInsert_v1 ( KWTrieIndex_v1 *self,
     bool proj, const char *str, uint32_t id )
 {
     rc_t rc;
 
     String key;
-    KTrieIdxNode_v1 *node;
+    KWTrieIdxNode_v1 *node;
 
 #if DISABLE_PROJ
     proj = false;
@@ -829,7 +745,7 @@ rc_t KTrieIndexInsert_v1 ( KTrieIndex_v1 *self,
         /* detect persisted data */
         if ( self -> pt . key2id != NULL )
         {
-            rc = KTrieIndexAttach_v1 ( self, proj, id );
+            rc = KWTrieIndexAttach_v1 ( self, proj, id );
             if ( rc != 0 )
                 return rc;
         }
@@ -882,7 +798,7 @@ rc_t KTrieIndexInsert_v1 ( KTrieIndex_v1 *self,
                 range = self -> last - id + 1;
                 if ( range > self -> len )
                 {
-                    rc = KTrieIndexExpandId2Node_v1 ( self, range );
+                    rc = KWTrieIndexExpandId2Node_v1 ( self, range );
                     if ( rc != 0 )
                     {
                         TrieUnlink ( & self -> key2id, & node -> n );
@@ -904,7 +820,7 @@ rc_t KTrieIndexInsert_v1 ( KTrieIndex_v1 *self,
                 range = id - self -> first + 1;
                 if ( range > self -> len )
                 {
-                    rc = KTrieIndexExpandId2Node_v1 ( self, range );
+                    rc = KWTrieIndexExpandId2Node_v1 ( self, range );
                     if ( rc != 0 )
                     {
                         TrieUnlink ( & self -> key2id, & node -> n );
@@ -932,12 +848,12 @@ rc_t KTrieIndexInsert_v1 ( KTrieIndex_v1 *self,
 }
 
 /* drop string from trie and all mappings */
-rc_t KTrieIndexDelete_v1 ( KTrieIndex_v1 *self, bool proj, const char *str )
+rc_t KWTrieIndexDelete_v1 ( KWTrieIndex_v1 *self, bool proj, const char *str )
 {
     rc_t rc;
 
     String key;
-    KTrieIdxNode_v1 *node;
+    KWTrieIdxNode_v1 *node;
 
 #if DISABLE_PROJ
     proj = 0;
@@ -949,7 +865,7 @@ rc_t KTrieIndexDelete_v1 ( KTrieIndex_v1 *self, bool proj, const char *str )
         /* detect persisted data */
         if ( self -> pt . key2id != NULL )
         {
-            rc = KTrieIndexAttach_v1 ( self, proj, 0 );
+            rc = KWTrieIndexAttach_v1 ( self, proj, 0 );
             if ( rc != 0 )
                 return rc;
         }
@@ -969,7 +885,7 @@ rc_t KTrieIndexDelete_v1 ( KTrieIndex_v1 *self, bool proj, const char *str )
        however, this implementation only allows unique
        mappings, so a simple find is sufficient */
     StringInitCString ( & key, str );
-    node = ( KTrieIdxNode_v1* ) TrieFind ( & self -> key2id, & key );
+    node = ( KWTrieIdxNode_v1* ) TrieFind ( & self -> key2id, & key );
     if ( node == NULL )
         return RC ( rcDB, rcIndex, rcRemoving, rcString, rcNotFound );
 
@@ -1007,7 +923,7 @@ rc_t KTrieIndexDelete_v1 ( KTrieIndex_v1 *self, bool proj, const char *str )
 }
 
 /* map key to id ( was Key2Id ) */
-rc_t KTrieIndexFind_v1 ( const KTrieIndex_v1 *self, const char *str, uint32_t *id,
+rc_t KWTrieIndexFind_v1 ( const KWTrieIndex_v1 *self, const char *str, uint32_t *id,
     int ( CC * custom_cmp ) ( const void *item, const PBSTNode *n, void *data ), void * data )
 {
     String key;
@@ -1030,10 +946,10 @@ rc_t KTrieIndexFind_v1 ( const KTrieIndex_v1 *self, const char *str, uint32_t *i
     }
     else
     {
-        const KTrieIdxNode_v1 *n;
+        const KWTrieIdxNode_v1 *n;
 
         StringInitCString ( & key, str );
-        n = ( const KTrieIdxNode_v1* ) TrieFind ( & self -> key2id, & key );
+        n = ( const KWTrieIdxNode_v1* ) TrieFind ( & self -> key2id, & key );
         if ( n != NULL )
         {
             * id = n -> id;
@@ -1045,7 +961,7 @@ rc_t KTrieIndexFind_v1 ( const KTrieIndex_v1 *self, const char *str, uint32_t *i
 }
 
 /* projection index id to key-string ( was Id2Key ) */
-rc_t KTrieIndexProject_v1 ( const KTrieIndex_v1 *self,
+rc_t KWTrieIndexProject_v1 ( const KWTrieIndex_v1 *self,
     uint32_t id, char *key_buff, size_t buff_size, size_t *actsize )
 {
     if ( self -> last < self -> first )
@@ -1066,7 +982,7 @@ rc_t KTrieIndexProject_v1 ( const KTrieIndex_v1 *self,
                 {
                     if (actsize)
                         *actsize = key -> size;
-                    
+
                     if ( key -> size >= buff_size )
                         rc = RC ( rcDB, rcIndex, rcProjecting, rcBuffer, rcInsufficient );
                     else
@@ -1084,7 +1000,7 @@ rc_t KTrieIndexProject_v1 ( const KTrieIndex_v1 *self,
              id >= self -> first &&
              id <= self -> last )
         {
-            const KTrieIdxNode_v1 *n = self -> id2node [ id - self -> first ];
+            const KWTrieIdxNode_v1 *n = self -> id2node [ id - self -> first ];
             if ( n != NULL )
             {
                 if ( n -> n . key . size >= buff_size )
