@@ -26,18 +26,21 @@
 
 #include <kdb/extern.h>
 
-#include "windex-priv.h"
-#include "trieidx-priv.h"
+#include "windex.h"
+#include "trieidx.h"
 
 #include <kdb/index.h>
+
 #include <kfs/directory.h>
 #include <kfs/file.h>
 #include <kfs/md5.h>
 #include <kfs/mmap.h>
+
 #include <klib/ptrie.h>
 #include <klib/text.h>
 #include <klib/pack.h>
 #include <klib/rc.h>
+
 #include <os-native.h>
 #include <sysalloc.h>
 
@@ -321,7 +324,7 @@ rc_t KPTrieIndexInitFromV1_v2 ( KPTrieIndex_v2 *self, const KMMap *mm, bool byte
         }
 
         if ( rc == 0 )
-        {        
+        {
             if ( i == self -> count )
                 return 0;
             rc = RC ( rcDB, rcIndex, rcConstructing, rcIndex, rcCorrupt );
@@ -333,7 +336,7 @@ rc_t KPTrieIndexInitFromV1_v2 ( KPTrieIndex_v2 *self, const KMMap *mm, bool byte
     return rc;
 }
 
-rc_t KPTrieIndexInit_v2 ( KPTrieIndex_v2 *self, const KMMap *mm, bool byteswap )
+rc_t KWPTrieIndexInit_v2 ( KPTrieIndex_v2 *self, const KMMap *mm, bool byteswap )
 {
     /* get size of map, assumed to be size of file */
     size_t size;
@@ -392,7 +395,7 @@ rc_t KPTrieIndexInit_v2 ( KPTrieIndex_v2 *self, const KMMap *mm, bool byteswap )
                     if ( ptsize == size )
                         return 0;
 
-                    /* calculate remaining bytes */                     
+                    /* calculate remaining bytes */
                     size -= ptsize;
 
                     /* there must be enough for an array of 4-byte node ids */
@@ -455,7 +458,7 @@ rc_t KPTrieIndexInit_v2 ( KPTrieIndex_v2 *self, const KMMap *mm, bool byteswap )
     return rc;
 }
 
-rc_t KPTrieIndexInit_v3_v4 ( KPTrieIndex_v2 *self, const KMMap *mm, bool byteswap, bool ptorig )
+rc_t KWPTrieIndexInit_v3_v4 ( KPTrieIndex_v2 *self, const KMMap *mm, bool byteswap, bool ptorig )
 {
     /* get size of map, assumed to be size of file */
     size_t size;
@@ -514,7 +517,7 @@ rc_t KPTrieIndexInit_v3_v4 ( KPTrieIndex_v2 *self, const KMMap *mm, bool byteswa
                     if ( ptsize == size )
                         return 0;
 
-                    /* calculate remaining bytes */                     
+                    /* calculate remaining bytes */
                     size -= ptsize;
 
                     /* there must be enough for an array of 4-byte node ids */
@@ -580,7 +583,7 @@ rc_t KPTrieIndexInit_v3_v4 ( KPTrieIndex_v2 *self, const KMMap *mm, bool byteswa
 /* Whack
  *  closes down keymap
  */
-void KPTrieIndexWhack_v2 ( KPTrieIndex_v2 *self )
+void KWPTrieIndexWhack_v2 ( KPTrieIndex_v2 *self )
 {
     free ( ( void* ) self -> id2ord . v8 );
     PTrieWhack ( self -> key2id );
@@ -588,7 +591,7 @@ void KPTrieIndexWhack_v2 ( KPTrieIndex_v2 *self )
     memset ( self, 0, sizeof * self );
 }
 
-uint32_t KPTrieIndexID2Ord_v2 ( const KPTrieIndex_v2 *self, int64_t id )
+uint32_t KWPTrieIndexID2Ord_v2 ( const KPTrieIndex_v2 *self, int64_t id )
 {
     if ( id >= self -> first && id <= self -> maxid )
     {
@@ -714,7 +717,7 @@ rc_t KPTrieIndexProject_v2 ( const KPTrieIndex_v2 *self,
 #endif
     char *key_buff, size_t buff_size, size_t *actsize )
 {
-    uint32_t nid, ord = KPTrieIndexID2Ord_v2 ( self, id );
+    uint32_t nid, ord = KWPTrieIndexID2Ord_v2 ( self, id );
     if ( ord != 0 )
     {
         assert ( start_id != NULL );
@@ -737,25 +740,25 @@ rc_t KPTrieIndexProject_v2 ( const KPTrieIndex_v2 *self,
             * start_id = self -> id2ord . v8 [ ord - 1 ];
             * span = ( uint32_t ) ( ( ( ord == self -> count ) ?
                 ( self -> maxid  - self -> first + 1 ) : self -> id2ord . v8 [ ord ] ) - * start_id );
-            *start_id += self->first; 
+            *start_id += self->first;
             break;
         case 2:
             * start_id = self -> id2ord . v16 [ ord - 1 ];
             * span = ( uint32_t ) ( ( ( ord == self -> count ) ?
                 ( self -> maxid  - self -> first + 1 ) : self -> id2ord . v16 [ ord ] ) - * start_id );
-            *start_id += self->first; 
+            *start_id += self->first;
             break;
         case 3:
             * start_id = self -> id2ord . v32 [ ord - 1 ];
             * span = ( uint32_t ) ( ( ( ord == self -> count ) ?
                 ( self -> maxid  - self -> first + 1 ) : self -> id2ord . v32 [ ord ] ) - * start_id );
-            *start_id += self->first; 
+            *start_id += self->first;
             break;
         case 4:
             * start_id = self -> id2ord . v64 [ ord - 1 ];
             * span = ( uint32_t ) ( ( ( ord == self -> count ) ?
                 ( self -> maxid  - self -> first + 1 ) : self -> id2ord . v64 [ ord ] ) - * start_id );
-            *start_id += self->first; 
+            *start_id += self->first;
             break;
         }
 
@@ -801,7 +804,7 @@ rc_t KPTrieIndexFind_v2 ( const KPTrieIndex_v2 *self,
 #endif
     int ( CC * custom_cmp ) ( const void *item, const PBSTNode *n, void *data ), void *data, bool convertFromV1 )
 {
-    rc_t rc;
+    rc_t rc = 0;
 
     /* detect empty index */
     if ( self -> count == 0 )
@@ -832,7 +835,7 @@ rc_t KPTrieIndexFind_v2 ( const KPTrieIndex_v2 *self,
                 * start_id = id;
                 rc = 0;
             }
-            else
+            else if ( self -> id_bits > 0 )
             {
                 /* should be native v2 */
                 rc = Unpack ( self -> id_bits, sizeof * start_id * 8,
@@ -840,13 +843,17 @@ rc_t KPTrieIndexFind_v2 ( const KPTrieIndex_v2 *self,
                     start_id, sizeof * start_id, & usize );
                 * start_id += self -> first;
             }
+            else
+            {
+                * start_id = self -> first;
+            }
 
             if ( rc == 0 )
             {
 #if V2FIND_RETURNS_SPAN
                 if ( self -> ord2node != NULL )
                 {
-                    uint32_t ord = KPTrieIndexID2Ord_v2 ( self, * start_id );
+                    uint32_t ord = KWPTrieIndexID2Ord_v2 ( self, * start_id );
                     if ( ord == 0 )
                         rc = RC ( rcDB, rcIndex, rcSelecting, rcId, rcNotFound );
                     else if ( ord == self -> count )
@@ -897,14 +904,14 @@ rc_t KPTrieIndexFind_v2 ( const KPTrieIndex_v2 *self,
  */
 
 static
-rc_t KTrieIdxNodeMake_v2_s1 ( KTrieIdxNode_v2_s1 **n, const String *key, int64_t id )
+rc_t KTrieIdxNodeMake_v2_s1 ( KWTrieIdxNode_v2_s1 **n, const String *key, int64_t id )
 {
     rc_t rc = TNodeMake ( ( TNode** ) n, sizeof ** n + key -> size );
     if ( rc != 0 )
         rc = ResetRCContext ( rc, rcDB, rcIndex, rcInserting );
     else
     {
-        KTrieIdxNode_v2_s1 *node = * n;
+        KWTrieIdxNode_v2_s1 *node = * n;
         string_copy ( node -> key, key -> size + 1, key -> addr, key -> size);
         StringInit ( & node -> n . key, node -> key, key -> size, key -> len );
         node -> start_id = id;
@@ -914,14 +921,14 @@ rc_t KTrieIdxNodeMake_v2_s1 ( KTrieIdxNode_v2_s1 **n, const String *key, int64_t
 }
 
 static
-rc_t KTrieIdxNodeMakeHole_v2_s1 ( KTrieIdxNode_v2_s1 **n, int64_t id )
+rc_t KTrieIdxNodeMakeHole_v2_s1 ( KWTrieIdxNode_v2_s1 **n, int64_t id )
 {
     rc_t rc = TNodeMake ( ( TNode** ) n, sizeof ** n );
     if ( rc != 0 )
         rc = ResetRCContext ( rc, rcDB, rcIndex, rcInserting );
     else
     {
-        KTrieIdxNode_v2_s1 *node = * n;
+        KWTrieIdxNode_v2_s1 *node = * n;
         node -> key [ 0 ] = 0;
         StringInit ( & node -> n . key, node -> key, 0, 0 );
         node -> start_id = id;
@@ -931,14 +938,14 @@ rc_t KTrieIdxNodeMakeHole_v2_s1 ( KTrieIdxNode_v2_s1 **n, int64_t id )
 }
 
 static
-rc_t KTrieIdxNodeMake_v2_s2 ( KTrieIdxNode_v2_s2 **n, const String *key, int64_t id )
+rc_t KTrieIdxNodeMake_v2_s2 ( KWTrieIdxNode_v2_s2 **n, const String *key, int64_t id )
 {
     rc_t rc = TNodeMake ( ( TNode** ) n, sizeof ** n + key -> size );
     if ( rc != 0 )
         rc = ResetRCContext ( rc, rcDB, rcIndex, rcInserting );
     else
     {
-        KTrieIdxNode_v2_s2 *node = * n;
+        KWTrieIdxNode_v2_s2 *node = * n;
         string_copy ( node -> key, key -> size + 1, key -> addr, key -> size);
         StringInit ( & node -> n . key, node -> key, key -> size, key -> len );
         node -> start_id = id;
@@ -964,11 +971,11 @@ void CC KTrieIdxNodeUnlink_v2 ( TNode *n, void *data )
 
 
 /*--------------------------------------------------------------------------
- * KTrieIndex_v2
+ * KWTrieIndex_v2
  */
 
 static
-uint32_t KTrieIndexID2Ord_v2 ( const KTrieIndex_v2 *self, int64_t id )
+uint32_t KWTrieIndexID2Ord_v2 ( const KWTrieIndex_v2 *self, int64_t id )
 {
     if ( id >= self -> first && id <= self -> last )
     {
@@ -976,7 +983,7 @@ uint32_t KTrieIndexID2Ord_v2 ( const KTrieIndex_v2 *self, int64_t id )
         for ( left = 1, right = count; left <= right; )
         {
             uint32_t ord = ( left + right ) >> 1;
-            const KTrieIdxNode_v2_s1 *node = self -> ord2node [ ord - 1 ];
+            const KWTrieIdxNode_v2_s1 *node = self -> ord2node [ ord - 1 ];
             if ( id == node -> start_id )
                 return ord;
 
@@ -1000,14 +1007,14 @@ uint32_t KTrieIndexID2Ord_v2 ( const KTrieIndex_v2 *self, int64_t id )
 }
 
 static
-uint32_t KTrieIndexNode2Ord_v2 ( const KTrieIndex_v2 *self, const KTrieIdxNode_v2_s1 *node )
+uint32_t KWTrieIndexNode2Ord_v2 ( const KWTrieIndex_v2 *self, const KWTrieIdxNode_v2_s1 *node )
 {
     if ( self -> ord2node != NULL )
-        return KTrieIndexID2Ord_v2 ( self, node -> start_id );
+        return KWTrieIndexID2Ord_v2 ( self, node -> start_id );
     return 0;
 }
 
-/* KTrieIndexWrite_v2
+/* KWTrieIndexWrite_v2
  */
 typedef struct PersistTrieData PersistTrieData;
 struct PersistTrieData
@@ -1028,7 +1035,7 @@ struct PersistTrieData
 };
 
 static
-rc_t CC KTrieIndexWrite_v2 ( void *param,
+rc_t CC KWTrieIndexWrite_v2 ( void *param,
     const void *buffer, size_t size, size_t *num_writ )
 {
     PersistTrieData *pb = param;
@@ -1076,10 +1083,10 @@ rc_t CC KTrieIndexWrite_v2 ( void *param,
     return 0;
 }
 
-/* KTrieIndexAux_v2
+/* KWTrieIndexAux_v2
  */
 static
-rc_t CC KTrieIndexAux_v2_s1 ( void *param, const void *node, size_t *num_writ,
+rc_t CC KWTrieIndexAux_v2_s1 ( void *param, const void *node, size_t *num_writ,
     PTWriteFunc write, void *write_param )
 {
     PersistTrieData *pb = param;
@@ -1087,7 +1094,7 @@ rc_t CC KTrieIndexAux_v2_s1 ( void *param, const void *node, size_t *num_writ,
     if ( write != NULL && pb -> node_data_size != 0 )
     {
         char buffer [ 8 ];
-        const KTrieIdxNode_v2_s1 *n = node;
+        const KWTrieIdxNode_v2_s1 *n = node;
 
         /* pack from 64 possible bits down to total id span */
         if ( pb -> id_bits != 0 )
@@ -1118,14 +1125,14 @@ rc_t CC KTrieIndexAux_v2_s1 ( void *param, const void *node, size_t *num_writ,
 }
 
 static
-rc_t CC KTrieIndexAux_v2_s2 ( void *param, const void *node, size_t *num_writ,
+rc_t CC KWTrieIndexAux_v2_s2 ( void *param, const void *node, size_t *num_writ,
     PTWriteFunc write, void *write_param )
 {
     PersistTrieData *pb = param;
 
     if ( write != NULL && pb -> node_data_size != 0 )
     {
-        const KTrieIdxNode_v2_s2 *n = node;
+        const KWTrieIdxNode_v2_s2 *n = node;
 
         rc_t rc;
         char buffer [ 12 ];
@@ -1167,13 +1174,13 @@ rc_t CC KTrieIndexAux_v2_s2 ( void *param, const void *node, size_t *num_writ,
     return 0;
 }
 
-/* KTrieIndexPersist_v*
+/* KWTrieIndexPersist_v*
  *  write keymap to indicated location
  */
 #if KDBINDEXVERS == 2
 
 static
-void KTrieIndexPersistHdr_v2 ( KTrieIndex_v2 *self, PersistTrieData *pb )
+void KWTrieIndexPersistHdr_v2 ( KWTrieIndex_v2 *self, PersistTrieData *pb )
 {
     KPTrieIndexHdr_v2 *hdr;
 
@@ -1217,7 +1224,7 @@ void KTrieIndexPersistHdr_v2 ( KTrieIndex_v2 *self, PersistTrieData *pb )
             if ( span > max_span )
                 max_span = span;
         }
-        
+
         span = ( uint32_t ) ( self -> last - prev );
         if ( span > max_span )
             max_span = span;
@@ -1245,7 +1252,7 @@ void KTrieIndexPersistHdr_v2 ( KTrieIndex_v2 *self, PersistTrieData *pb )
 #else
 
 static
-void KTrieIndexPersistHdr_v3_v4 ( KTrieIndex_v2 *self, PersistTrieData *pb )
+void KWTrieIndexPersistHdr_v3_v4 ( KWTrieIndex_v2 *self, PersistTrieData *pb )
 {
     KPTrieIndexHdr_v3 *hdr;
 
@@ -1290,7 +1297,7 @@ void KTrieIndexPersistHdr_v3_v4 ( KTrieIndex_v2 *self, PersistTrieData *pb )
             if ( span > max_span )
                 max_span = span;
         }
-        
+
         span = ( uint32_t ) ( self -> last - prev );
         if ( span > max_span )
             max_span = span;
@@ -1315,10 +1322,10 @@ void KTrieIndexPersistHdr_v3_v4 ( KTrieIndex_v2 *self, PersistTrieData *pb )
     hdr -> align [ 0 ] = hdr -> align [ 1 ] = 0;
 }
 
-#endif 
+#endif
 
 static
-rc_t KTrieIndexPersistTrie_v2 ( const KTrieIndex_v2 *self, PersistTrieData *pb )
+rc_t KWTrieIndexPersistTrie_v2 ( const KWTrieIndex_v2 *self, PersistTrieData *pb )
 {
     rc_t rc;
 
@@ -1329,13 +1336,13 @@ rc_t KTrieIndexPersistTrie_v2 ( const KTrieIndex_v2 *self, PersistTrieData *pb )
     {
         pb -> node_data_size = ( pb -> id_bits + 7 ) >> 3;
         rc = TriePersist ( & self -> key2id, & pb -> ptt_size,
-            false, KTrieIndexWrite_v2, pb, KTrieIndexAux_v2_s1, pb );
+            false, KWTrieIndexWrite_v2, pb, KWTrieIndexAux_v2_s1, pb );
     }
     else
     {
         pb -> node_data_size = ( pb -> id_bits + pb -> span_bits + 7 ) >> 3;
         rc = TriePersist ( & self -> key2id, & pb -> ptt_size,
-            false, KTrieIndexWrite_v2, pb, KTrieIndexAux_v2_s2, pb );
+            false, KWTrieIndexWrite_v2, pb, KWTrieIndexAux_v2_s2, pb );
     }
 
     if ( rc == 0 && pb -> marker != 0 )
@@ -1352,14 +1359,14 @@ rc_t KTrieIndexPersistTrie_v2 ( const KTrieIndex_v2 *self, PersistTrieData *pb )
 
 
 static
-rc_t KTrieIndexPersistProjContig_v2 ( const KTrieIndex_v2 *self,
+rc_t KWTrieIndexPersistProjContig_v2 ( const KWTrieIndex_v2 *self,
     PersistTrieData *pb, PTrie *tt, uint32_t *ord2node )
 {
     uint32_t i, j, nid;
     int64_t id = self -> first;
     for ( i = j = nid = 0; i < self -> count; ++ id, ++ j, ++ i )
     {
-        const KTrieIdxNode_v2_s1 *node = self -> ord2node [ i ];
+        const KWTrieIdxNode_v2_s1 *node = self -> ord2node [ i ];
 
         /* back fill repeats */
         for ( ; id < node -> start_id; ++ id )
@@ -1388,14 +1395,14 @@ rc_t KTrieIndexPersistProjContig_v2 ( const KTrieIndex_v2 *self,
 }
 
 static
-rc_t KTrieIndexPersistProjSparse_v2 ( const KTrieIndex_v2 *self,
+rc_t KWTrieIndexPersistProjSparse_v2 ( const KWTrieIndex_v2 *self,
     PersistTrieData *pb, PTrie *tt, uint32_t *ord2node, bitsz_t *psize )
 {
     uint32_t i, nid;
     int64_t *id2ord = ( void* ) & ord2node [ self -> count ];
     for ( i = 0; i < self -> count; ++ i )
     {
-        const KTrieIdxNode_v2_s1 *node = self -> ord2node [ i ];
+        const KWTrieIdxNode_v2_s1 *node = self -> ord2node [ i ];
 
         /* record negated id for i - see 1st derivative below */
         id2ord [ i ] = - node -> start_id;
@@ -1446,7 +1453,7 @@ rc_t KTrieIndexPersistProjSparse_v2 ( const KTrieIndex_v2 *self,
 #if KDBINDEXVERS == 2
 
 static
-rc_t KTrieIndexPersistProj_v2 ( const KTrieIndex_v2 *self, PersistTrieData *pb )
+rc_t KWTrieIndexPersistProj_v2 ( const KWTrieIndex_v2 *self, PersistTrieData *pb )
 {
     rc_t rc = 0;
     void * addr;
@@ -1541,16 +1548,16 @@ rc_t KTrieIndexPersistProj_v2 ( const KTrieIndex_v2 *self, PersistTrieData *pb )
             assert ( self -> count >= PTrieCount ( tt ) );
             ord2node = ( void* ) ( ( char* ) addr + pb -> ptt_size );
             assert ( ( ( size_t ) ord2node & 3 ) == 0 );
-            
+
             /* set count */
             * ord2node ++ = self -> count;
 
             if ( ! is_sparse )
-                rc = KTrieIndexPersistProjContig_v2 ( self, pb, tt, ord2node );
+                rc = KWTrieIndexPersistProjContig_v2 ( self, pb, tt, ord2node );
             else
             {
                 bitsz_t psize;
-                rc = KTrieIndexPersistProjSparse_v2 ( self, pb, tt, ord2node, & psize );
+                rc = KWTrieIndexPersistProjSparse_v2 ( self, pb, tt, ord2node, & psize );
                 if ( rc == 0 )
                 {
                     map_size -= ( size_t ) self -> count << 3;
@@ -1561,7 +1568,7 @@ rc_t KTrieIndexPersistProj_v2 ( const KTrieIndex_v2 *self, PersistTrieData *pb )
             /* done with pttree */
             PTrieWhack ( tt );
         }
-        rc = KFileWrite ( pb -> f, file_size, 
+        rc = KFileWrite ( pb -> f, file_size,
 			  (uint8_t*)addr + num_to_read,  map_size - num_to_read, & num_writ );
         if ( rc == 0  &&  num_writ != map_size - num_to_read )
             rc = RC ( rcDB, rcIndex, rcPersisting, rcHeader, rcInsufficient );
@@ -1574,10 +1581,10 @@ rc_t KTrieIndexPersistProj_v2 ( const KTrieIndex_v2 *self, PersistTrieData *pb )
 #else
 
 static
-rc_t KTrieIndexPersistProj_v3 ( const KTrieIndex_v2 *self, PersistTrieData *pb )
+rc_t KWTrieIndexPersistProj_v3 ( const KWTrieIndex_v2 *self, PersistTrieData *pb )
 {
     rc_t rc = 0;
-    void * addr;
+    void * addr = NULL;
     size_t map_size;
     uint64_t file_size;
     size_t num_to_read;
@@ -1670,16 +1677,16 @@ rc_t KTrieIndexPersistProj_v3 ( const KTrieIndex_v2 *self, PersistTrieData *pb )
             assert ( self -> count >= PTrieCount ( tt ) );
             ord2node = ( void* ) ( ( char* ) addr + pb -> ptt_size );
             assert ( ( ( size_t ) ord2node & 3 ) == 0 );
-            
+
             /* set count */
             * ord2node ++ = self -> count;
 
             if ( ! is_sparse )
-                rc = KTrieIndexPersistProjContig_v2 ( self, pb, tt, ord2node );
+                rc = KWTrieIndexPersistProjContig_v2 ( self, pb, tt, ord2node );
             else
             {
                 bitsz_t psize;
-                rc = KTrieIndexPersistProjSparse_v2 ( self, pb, tt, ord2node, & psize );
+                rc = KWTrieIndexPersistProjSparse_v2 ( self, pb, tt, ord2node, & psize );
                 if ( rc == 0 )
                 {
                     map_size -= ( size_t ) self -> count << 3;
@@ -1692,7 +1699,7 @@ rc_t KTrieIndexPersistProj_v3 ( const KTrieIndex_v2 *self, PersistTrieData *pb )
 
             if ( rc == 0 )
             {
-                rc = KFileWrite ( pb -> f, file_size, 
+                rc = KFileWrite ( pb -> f, file_size,
                      ( uint8_t* ) addr + num_to_read,  map_size - num_to_read, & num_writ );
 
                 if ( rc == 0  &&  num_writ != map_size - num_to_read )
@@ -1709,7 +1716,7 @@ rc_t KTrieIndexPersistProj_v3 ( const KTrieIndex_v2 *self, PersistTrieData *pb )
 #endif
 
 static
-rc_t KTrieIndexCreateMD5Wrapper ( KDirectory *dir, KFile ** fp, KMD5File ** wrapper,
+rc_t KWTrieIndexCreateMD5Wrapper ( KDirectory *dir, KFile ** fp, KMD5File ** wrapper,
     char relpath [ 256 ], const char md5_relpath [ 260 ] )
 {
     /* create the md5 file for read/write */
@@ -1766,7 +1773,7 @@ rc_t KTrieIndexCreateMD5Wrapper ( KDirectory *dir, KFile ** fp, KMD5File ** wrap
     return rc;
 }
 
-rc_t KTrieIndexPersist_v2 ( const KTrieIndex_v2 *self,
+rc_t KWTrieIndexPersist_v2 ( const KWTrieIndex_v2 *self,
     bool proj, KDirectory *dir, const char *path, bool use_md5 )
 {
     rc_t rc;
@@ -1804,33 +1811,33 @@ rc_t KTrieIndexPersist_v2 ( const KTrieIndex_v2 *self,
             {
                 /* if using md5, wrap output file */
                 if ( use_md5 )
-                    rc = KTrieIndexCreateMD5Wrapper ( dir, & pb . f, & pb . fmd5, tmpname, tmpmd5name );
+                    rc = KWTrieIndexCreateMD5Wrapper ( dir, & pb . f, & pb . fmd5, tmpname, tmpmd5name );
                 if ( rc == 0 )
                 {
                     /* initial size */
                     pb . ptt_size = 0;
 #if KDBINDEXVERS == 2
-                    KTrieIndexPersistHdr_v2 ( ( KTrieIndex_v2* ) self, & pb );
+                    KWTrieIndexPersistHdr_v2 ( ( KWTrieIndex_v2* ) self, & pb );
 #else
-                    KTrieIndexPersistHdr_v3_v4 ( ( KTrieIndex_v2* ) self, & pb );
+                    KWTrieIndexPersistHdr_v3_v4 ( ( KWTrieIndex_v2* ) self, & pb );
 #endif
 
                     /* persist tree */
-                    rc = KTrieIndexPersistTrie_v2 ( self, & pb );
+                    rc = KWTrieIndexPersistTrie_v2 ( self, & pb );
                     if ( rc == 0 )
                     {
                         /* persist projection table */
                         if ( proj )
                         {
 #if KDBINDEXVERS == 2
-                            rc = KTrieIndexPersistProj_v2 ( self, & pb );
+                            rc = KWTrieIndexPersistProj_v2 ( self, & pb );
 #else
-                            rc = KTrieIndexPersistProj_v3 ( self, & pb );
+                            rc = KWTrieIndexPersistProj_v3 ( self, & pb );
 #endif
                         }
                     }
                 }
-                    
+
                 /* close down the file now, success or not */
                 KFileRelease ( pb . f );
                 pb . f = NULL;
@@ -1843,7 +1850,7 @@ rc_t KTrieIndexPersist_v2 ( const KTrieIndex_v2 *self,
                 if ( rc == 0 )
                 {
                     /* works even if "path" is absolute */
-                    rc = KDirectoryRename ( dir, false, tmpname, path );                        
+                    rc = KDirectoryRename ( dir, false, tmpname, path );
                     if ( rc == 0 )
                     {
                         int tmplen;
@@ -1879,21 +1886,21 @@ rc_t KTrieIndexPersist_v2 ( const KTrieIndex_v2 *self,
         /* douse buffer */
         free ( pb . buffer );
     }
-    
+
     return rc;
 }
 
 
 /* whack whack */
-void KTrieIndexWhack_v2 ( KTrieIndex_v2 *self )
+void KWTrieIndexWhack_v2 ( KWTrieIndex_v2 *self )
 {
-    KPTrieIndexWhack_v2 ( & self -> pt );
+    KWPTrieIndexWhack_v2 ( & self -> pt );
     TrieWhack ( & self -> key2id, KTrieIdxNodeWhack_v2, NULL );
     free ( self -> ord2node );
 }
 
 /* initialize an index from file - can be NULL */
-rc_t KTrieIndexOpen_v2 ( KTrieIndex_v2 *self, const KMMap *mm, bool byteswap )
+rc_t KWTrieIndexOpen_v2 ( KWTrieIndex_v2 *self, const KMMap *mm, bool byteswap )
 {
     rc_t rc;
     bool ptorig = false;
@@ -1928,12 +1935,12 @@ rc_t KTrieIndexOpen_v2 ( KTrieIndex_v2 *self, const KMMap *mm, bool byteswap )
         rc = KPTrieIndexInitFromV1_v2 ( & self -> pt, mm, byteswap );
         break;
     case 2:
-        rc = KPTrieIndexInit_v2 ( & self -> pt, mm, byteswap );
+        rc = KWPTrieIndexInit_v2 ( & self -> pt, mm, byteswap );
         break;
     case 3:
         ptorig = true;
     case 4:
-        rc = KPTrieIndexInit_v3_v4 ( & self -> pt, mm, byteswap, ptorig );
+        rc = KWPTrieIndexInit_v3_v4 ( & self -> pt, mm, byteswap, ptorig );
         break;
     default:
         rc = RC(rcDB, rcIndex, rcConstructing, rcIndex, rcBadVersion);
@@ -1960,18 +1967,18 @@ rc_t KTrieIndexOpen_v2 ( KTrieIndex_v2 *self, const KMMap *mm, bool byteswap )
         /* self -> pt gets whacked below */
     }
 
-    KTrieIndexWhack_v2 ( self );
+    KWTrieIndexWhack_v2 ( self );
     return rc;
 }
 
-/* KTrieIndexPopulate_v2
+/* KWTrieIndexPopulate_v2
  */
-typedef struct KTrieIndexPopulateData_v2_s2 KTrieIndexPopulateData_v2_s2;
-struct KTrieIndexPopulateData_v2_s2
+typedef struct KWTrieIndexPopulateData_v2_s2 KWTrieIndexPopulateData_v2_s2;
+struct KWTrieIndexPopulateData_v2_s2
 {
     int64_t first;
 
-    KTrieIndex_v2 *self;
+    KWTrieIndex_v2 *self;
     uint32_t count;
     rc_t rc;
 
@@ -1980,10 +1987,10 @@ struct KTrieIndexPopulateData_v2_s2
 };
 
 static
-bool CC KTrieIndexPopulate_v2_s2 ( PTNode *n, void *data )
+bool CC KWTrieIndexPopulate_v2_s2 ( PTNode *n, void *data )
 {
     const String *key;
-    KTrieIndexPopulateData_v2_s2 *pb = data;
+    KWTrieIndexPopulateData_v2_s2 *pb = data;
 
     int64_t id;
     size_t usize;
@@ -2004,7 +2011,7 @@ bool CC KTrieIndexPopulate_v2_s2 ( PTNode *n, void *data )
     pb -> rc = PTNodeMakeKey ( n, & key );
     if ( pb -> rc == 0 )
     {
-        KTrieIdxNode_v2_s2 *node;
+        KWTrieIdxNode_v2_s2 *node;
         pb -> rc = KTrieIdxNodeMake_v2_s2 ( & node, key, id + pb -> first );
         StringWhack ( ( String* ) key );
         if ( pb -> rc == 0 )
@@ -2025,11 +2032,11 @@ bool CC KTrieIndexPopulate_v2_s2 ( PTNode *n, void *data )
     return true;
 }
 
-/* KTrieIndexAttach_v2
+/* KWTrieIndexAttach_v2
  *  attach a keymap to an existing table
  */
 static
-rc_t KTrieIndexPopulate_v2_s1 ( KTrieIndex_v2 *self, uint32_t i, int64_t idd )
+rc_t KWTrieIndexPopulate_v2_s1 ( KWTrieIndex_v2 *self, uint32_t i, int64_t idd )
 {
     rc_t rc;
     uint32_t nid = self -> pt . ord2node [ i ];
@@ -2073,7 +2080,7 @@ rc_t KTrieIndexPopulate_v2_s1 ( KTrieIndex_v2 *self, uint32_t i, int64_t idd )
     return rc;
 }
 
-rc_t KTrieIndexAttach_v2 ( KTrieIndex_v2 *self, bool proj )
+rc_t KWTrieIndexAttach_v2 ( KWTrieIndex_v2 *self, bool proj )
 {
     rc_t rc = 0;
 
@@ -2100,23 +2107,23 @@ rc_t KTrieIndexAttach_v2 ( KTrieIndex_v2 *self, bool proj )
         {
         case 0:  /* 1-1 id to name */
             for ( rc = 0, i = 0; i < self -> pt . count && rc == 0; ++ i )
-                rc = KTrieIndexPopulate_v2_s1 ( self, i, i );
+                rc = KWTrieIndexPopulate_v2_s1 ( self, i, i );
             break;
         case 1:  /* sparse 8-bit   */
             for ( rc = 0, i = 0; i < self -> pt . count && rc == 0; ++ i )
-                rc = KTrieIndexPopulate_v2_s1 ( self, i, self -> pt . id2ord . v8 [ i ] );
+                rc = KWTrieIndexPopulate_v2_s1 ( self, i, self -> pt . id2ord . v8 [ i ] );
             break;
         case 2:  /* sparse 16-bit  */
             for ( rc = 0, i = 0; i < self -> pt . count && rc == 0; ++ i )
-                rc = KTrieIndexPopulate_v2_s1 ( self, i, self -> pt . id2ord . v16 [ i ] );
+                rc = KWTrieIndexPopulate_v2_s1 ( self, i, self -> pt . id2ord . v16 [ i ] );
             break;
         case 3:  /* sparse 32-bit  */
             for ( rc = 0, i = 0; i < self -> pt . count && rc == 0; ++ i )
-                rc = KTrieIndexPopulate_v2_s1 ( self, i, self -> pt . id2ord . v32 [ i ] );
+                rc = KWTrieIndexPopulate_v2_s1 ( self, i, self -> pt . id2ord . v32 [ i ] );
             break;
         case 4:  /* sparse 64-bit  */
             for ( rc = 0, i = 0; i < self -> pt . count && rc == 0; ++ i )
-                rc = KTrieIndexPopulate_v2_s1 ( self, i, self -> pt . id2ord . v64 [ i ] );
+                rc = KWTrieIndexPopulate_v2_s1 ( self, i, self -> pt . id2ord . v64 [ i ] );
             break;
         }
 
@@ -2130,12 +2137,12 @@ rc_t KTrieIndexAttach_v2 ( KTrieIndex_v2 *self, bool proj )
     }
     else
     {
-        KTrieIndexPopulateData_v2_s2 pb;
+        KWTrieIndexPopulateData_v2_s2 pb;
         pb . first = self -> pt . first;
         pb . self = self;
         pb . count = 0;
         pb . rc = 0;
-        PTrieDoUntil ( self -> pt . key2id, KTrieIndexPopulate_v2_s2, & pb );
+        PTrieDoUntil ( self -> pt . key2id, KWTrieIndexPopulate_v2_s2, & pb );
         if ( pb . rc == 0 && pb . count != self -> pt . count )
             return RC ( rcDB, rcIndex, rcUpdating, rcIndex, rcCorrupt );
         self -> count = pb . count;
@@ -2146,12 +2153,12 @@ rc_t KTrieIndexAttach_v2 ( KTrieIndex_v2 *self, bool proj )
     self -> last = self -> pt . last;
 
     /* should be able to drop persisted copy now */
-    KPTrieIndexWhack_v2 ( & self -> pt );
+    KWPTrieIndexWhack_v2 ( & self -> pt );
 
     return 0;
 }
 
-rc_t KTrieIndexInsert_v2 ( KTrieIndex_v2 *self,
+rc_t KWTrieIndexInsert_v2 ( KWTrieIndex_v2 *self,
     bool proj, const char *str, int64_t id )
 {
     rc_t rc;
@@ -2181,7 +2188,7 @@ rc_t KTrieIndexInsert_v2 ( KTrieIndex_v2 *self,
         if ( self -> pt . key2id != NULL )
         {
             /* load persisted data into core */
-            rc = KTrieIndexAttach_v2 ( self, proj );
+            rc = KWTrieIndexAttach_v2 ( self, proj );
             if ( rc != 0 )
                 return rc;
 
@@ -2203,7 +2210,7 @@ rc_t KTrieIndexInsert_v2 ( KTrieIndex_v2 *self,
     /* insertion strategy depends upon projection index */
     if ( proj )
     {
-        KTrieIdxNode_v2_s1 *node;
+        KWTrieIdxNode_v2_s1 *node;
 
         /* check for extension of last node */
         if ( count != 0 )
@@ -2303,14 +2310,14 @@ rc_t KTrieIndexInsert_v2 ( KTrieIndex_v2 *self,
     }
     else
     {
-        KTrieIdxNode_v2_s2 *node;
+        KWTrieIdxNode_v2_s2 *node;
 
         /* make a new mapping starting with id and a span of 1 */
         rc = KTrieIdxNodeMake_v2_s2 ( & node, & key, id );
         if ( rc == 0 )
         {
             /* attempt insertion */
-            KTrieIdxNode_v2_s2 *exist;
+            KWTrieIdxNode_v2_s2 *exist;
             rc = TrieInsertUnique ( & self -> key2id, & node -> n, ( TNode** ) & exist );
             if ( rc == 0 )
             {
@@ -2358,7 +2365,7 @@ rc_t KTrieIndexInsert_v2 ( KTrieIndex_v2 *self,
 }
 
 /* drop string from trie and all mappings */
-rc_t KTrieIndexDelete_v2 ( KTrieIndex_v2 *self, bool proj, const char *str )
+rc_t KWTrieIndexDelete_v2 ( KWTrieIndex_v2 *self, bool proj, const char *str )
 {
     rc_t rc;
     String key;
@@ -2376,7 +2383,7 @@ rc_t KTrieIndexDelete_v2 ( KTrieIndex_v2 *self, bool proj, const char *str )
         /* detect persisted data */
         if ( self -> pt . key2id != NULL )
         {
-            rc = KTrieIndexAttach_v2 ( self, proj );
+            rc = KWTrieIndexAttach_v2 ( self, proj );
             if ( rc != 0 )
                 return rc;
         }
@@ -2397,8 +2404,8 @@ rc_t KTrieIndexDelete_v2 ( KTrieIndex_v2 *self, bool proj, const char *str )
     /* neutralize node in projection index */
     if ( proj )
     {
-        KTrieIdxNode_v2_s1 *node = ( KTrieIdxNode_v2_s1* ) tnode;
-        uint32_t ord = KTrieIndexNode2Ord_v2 ( self, node );
+        KWTrieIdxNode_v2_s1 *node = ( KWTrieIdxNode_v2_s1* ) tnode;
+        uint32_t ord = KWTrieIndexNode2Ord_v2 ( self, node );
         if ( ord != 0 )
         {
             self -> ord2node [ ord - 1 ] -> n . key . size = 0;
@@ -2415,7 +2422,7 @@ rc_t KTrieIndexDelete_v2 ( KTrieIndex_v2 *self, bool proj, const char *str )
 }
 
 /* map key to id range */
-rc_t KTrieIndexFind_v2 ( const KTrieIndex_v2 *self,
+rc_t KWTrieIndexFind_v2 ( const KWTrieIndex_v2 *self,
     const char *str, int64_t *start_id,
 #if V2FIND_RETURNS_SPAN
     uint32_t *span,
@@ -2435,8 +2442,8 @@ rc_t KTrieIndexFind_v2 ( const KTrieIndex_v2 *self,
         {
             if ( self -> ord2node != NULL )
             {
-                const KTrieIdxNode_v2_s1 *node = ( const KTrieIdxNode_v2_s1* ) tnode;
-                uint32_t ord = KTrieIndexNode2Ord_v2 ( self, node );
+                const KWTrieIdxNode_v2_s1 *node = ( const KWTrieIdxNode_v2_s1* ) tnode;
+                uint32_t ord = KWTrieIndexNode2Ord_v2 ( self, node );
                 if ( ord == 0 )
                     return RC ( rcDB, rcIndex, rcSelecting, rcIndex, rcCorrupt );
 
@@ -2450,7 +2457,7 @@ rc_t KTrieIndexFind_v2 ( const KTrieIndex_v2 *self,
             }
             else
             {
-                const KTrieIdxNode_v2_s2 *node = ( const KTrieIdxNode_v2_s2* ) tnode;
+                const KWTrieIdxNode_v2_s2 *node = ( const KWTrieIdxNode_v2_s2* ) tnode;
                 * start_id = node -> start_id;
 #if V2FIND_RETURNS_SPAN
                 * span = node -> span;
@@ -2475,18 +2482,18 @@ rc_t KTrieIndexFind_v2 ( const KTrieIndex_v2 *self,
 }
 
 /* projection index id to key-string */
-typedef struct KTrieIndexProjectData_v2 KTrieIndexProjectData_v2;
-struct KTrieIndexProjectData_v2
+typedef struct KWTrieIndexProjectData_v2 KWTrieIndexProjectData_v2;
+struct KWTrieIndexProjectData_v2
 {
     int64_t id;
-    const KTrieIdxNode_v2_s2 *node;
+    const KWTrieIdxNode_v2_s2 *node;
 };
 
 static
-bool CC KTrieIndexProjectScan_v2 ( TNode *n, void *data )
+bool CC KWTrieIndexProjectScan_v2 ( TNode *n, void *data )
 {
-    KTrieIndexProjectData_v2 *pb = (KTrieIndexProjectData_v2 *)data;
-    const KTrieIdxNode_v2_s2 *node = ( const KTrieIdxNode_v2_s2* ) n;
+    KWTrieIndexProjectData_v2 *pb = (KWTrieIndexProjectData_v2 *)data;
+    const KWTrieIdxNode_v2_s2 *node = ( const KWTrieIdxNode_v2_s2* ) n;
 
     if ( pb -> id >= node -> start_id &&
          pb -> id < node -> start_id + node -> span )
@@ -2498,7 +2505,7 @@ bool CC KTrieIndexProjectScan_v2 ( TNode *n, void *data )
     return false;
 }
 
-rc_t KTrieIndexProject_v2 ( const KTrieIndex_v2 *self,
+rc_t KWTrieIndexProject_v2 ( const KWTrieIndex_v2 *self,
     int64_t id,
 #if V2FIND_RETURNS_SPAN
      int64_t *start_id, uint32_t *span,
@@ -2509,11 +2516,11 @@ rc_t KTrieIndexProject_v2 ( const KTrieIndex_v2 *self,
     {
         if ( self -> ord2node != NULL )
         {
-            uint32_t ord = KTrieIndexID2Ord_v2 ( self, id );
+            uint32_t ord = KWTrieIndexID2Ord_v2 ( self, id );
             if ( ord != 0 )
             {
-                const KTrieIdxNode_v2_s1 *node = self -> ord2node [ ord - 1 ];
-                
+                const KWTrieIdxNode_v2_s1 *node = self -> ord2node [ ord - 1 ];
+
                 if (actsize)
                     *actsize = node -> n . key . size;
                 if ( node -> n . key . size >= buff_size )
@@ -2528,12 +2535,12 @@ rc_t KTrieIndexProject_v2 ( const KTrieIndex_v2 *self,
         }
         else
         {
-            KTrieIndexProjectData_v2 pb;
+            KWTrieIndexProjectData_v2 pb;
             pb . id = id;
-            if ( TrieDoUntil ( & self -> key2id, KTrieIndexProjectScan_v2, & pb ) )
+            if ( TrieDoUntil ( & self -> key2id, KWTrieIndexProjectScan_v2, & pb ) )
             {
-                const KTrieIdxNode_v2_s2 *node = pb . node;
-                
+                const KWTrieIdxNode_v2_s2 *node = pb . node;
+
                 if (actsize)
                     *actsize = node -> n . key . size;
                 if ( node -> n . key . size >= buff_size )
