@@ -65,6 +65,9 @@
 #include <errno.h>
 #include <assert.h>
 
+#ifndef __USE_GNU
+#define __USE_GNU
+#endif
 #include <dlfcn.h>
 
 
