@@ -79,6 +79,11 @@ static String * s_LoadedQuality = NULL; /* default or from configuration*/
  *  opaque handle to library
  */
 
+void VDBManagerWhackStatic ( void ) {
+    StringWhack ( s_LoadedQuality );
+    s_LoadedQuality = NULL;
+}
+
 /* Whack
  */
 static
@@ -102,9 +107,8 @@ rc_t VDBManagerWhack ( VDBManager *self )
         VSchemaRelease ( self -> schema );
         VLinkerRelease ( self -> linker );
 	
-        StringWhack ( s_LoadedQuality );
-        s_LoadedQuality = NULL;
-	
+        VDBManagerWhackStatic ();
+
         free ( self );
         return 0;
     }
@@ -644,7 +648,7 @@ LIB_EXPORT rc_t CC VDBManagerGetObjVersion ( const VDBManager *self, ver_t * ver
         {
             const KTable *tbl;
             const KDatabase *db;
-            const KMetadata *meta;
+            const KMetadata *meta = NULL;
 
             int path_type = KDBManagerPathType ( self -> kmgr, "%s", path ) & ~ kptAlias;
             switch ( path_type )
