@@ -27,6 +27,7 @@
 #include <klib/log.h> /* PLOGERR */
 #include <klib/printf.h> /* string_printf */
 #include <klib/rc.h> /* RC */
+#include <klib/text.h>
 
 #include <vfs/manager.h> /* VFSManagerMake */
 #include <vfs/path.h> /* VPath */
@@ -608,7 +609,7 @@ rc_t ItemAddFormat ( Item * self, const char * cType, const Data * dad,
                     break;
                 }
             }
-        else { 
+        else {
             assert(self->nElm);
             if (self->elm[self->nElm - 1].path[0] == NULL)
                 idx = self->nElm - 1;
@@ -636,7 +637,7 @@ rc_t ItemAddFormat ( Item * self, const char * cType, const Data * dad,
 
     if ( elm -> cType == NULL ) {
         if ( cType != NULL ) {
-            elm -> cType = strdup ( cType );
+            elm -> cType = string_dup_measure ( cType, NULL );
             if ( elm -> cType == NULL )
                 return RC ( rcVFS, rcQuery, rcExecuting,
                                    rcMemory, rcExhausted );
@@ -646,7 +647,7 @@ rc_t ItemAddFormat ( Item * self, const char * cType, const Data * dad,
 
     if (elm->name == NULL ) {
         if ( name != NULL) {
-            elm->name = strdup(name);
+            elm->name = string_dup_measure(name, NULL);
             if (elm->name == NULL)
                 return RC(rcVFS, rcQuery, rcExecuting,
                                  rcMemory, rcExhausted);
@@ -1613,6 +1614,7 @@ static rc_t FileAddLinks ( File * self, const KJsonObject * node,
         if (THRESHOLD > THRESHOLD_NO_DEBUG)
             DBGMSG ( DBG_VFS, DBG_FLAG ( DBG_VFS_JSON ),
                 ( "... %s: cannot find any link\n", error ) );
+        UNUSED(error);
     }
 
     return rc;
@@ -1991,9 +1993,10 @@ void ContainerProcessStatus(Container * self, const Data * data) {
         /* log message to user */
         if (logError)
             PLOGERR(lvl, (lvl, self->rc,
-                "failed to resolve accession '$(acc)' - $(msg) ( $(code) )",
-                "acc=%s,msg=%s,code=%u",
-                data->acc, self->status.msg, self->status.code));
+                "failed to resolve accession '$(acc)' - $(msg) ( $(code) ); "
+                "ncbi_phid=$(phid)",
+                "acc=%s,msg=%s,code=%u,phid=%s",
+                data->acc, self->status.msg, self->status.code, data->phid));
     }
 }
 
@@ -2412,7 +2415,12 @@ rc_t KSrvRespObjRelease ( const KSrvRespObj * cself ) {
 rc_t KSrvRespObjGetAccOrId(const KSrvRespObj * self,
     const char ** acc, uint32_t * id)
 {
-    assert( self && self ->obj && acc && id );
+    if (acc == NULL || id == NULL)
+        return RC(rcVFS, rcQuery, rcExecuting, rcParam, rcNull);
+    if (self == NULL)
+        return RC(rcVFS, rcQuery, rcExecuting, rcSelf, rcNull);
+
+    assert(self->obj);
 
     *acc = self->obj->acc;
     *id = self->obj->id;
