@@ -30,6 +30,8 @@
 #include <kfg/properties.h> // KConfig_Set_Report_Cloud_Instance_Identity
 
 #include <klib/data-buffer.h>
+#include <klib/printf.h> // string_printf
+#include <klib/vdb_release_version.h> // VDB_RELEASE_VERSION
 
 #include <kns/http.h>
 #include <kns/http-priv.h>
@@ -44,6 +46,8 @@
 
 #include <ktst/unit_test.hpp>
 
+#include <algorithm>
+
 #define ALL
 
 static rc_t argsHandler ( int argc, char * argv [] );
@@ -94,6 +98,71 @@ FIXTURE_TEST_CASE(HttpRequest_POST_NoParams, HttpRequestFixture)
 }
 #endif
 
+FIXTURE_TEST_CASE(HttpRequest_PUT_sra, HttpRequestFixture)
+{
+    MakeRequest( GetName() );
+
+    KClientHttpResult *rslt;
+    TestStream::AddResponse("HTTP/1.1 200 OK\r\n");
+    REQUIRE_RC ( KClientHttpRequestPUT ( m_req, & rslt, true ) ); // format for SRA
+    REQUIRE_EQ( size_t(1), TestStream::m_requests.size() );
+
+    char version[16] = "";
+    REQUIRE_RC(string_printf(
+        version, sizeof version, NULL, "%V", VDB_RELEASE_VERSION));
+    assert(m_req && m_req->http);
+    string expected("PUT ");
+    if (m_req->http->uf == eUFAbsolute)
+        expected += "http://HttpRequest_PUT_sra.com";
+    expected +=
+        "/blah HTTP/1.1\r\n"
+        "Host: HttpRequest_PUT_sra.com\r\n"
+        "Accept: */*\r\n"
+        "X-SRA-Release: " + string(version) + "\r\n"
+        "X-VDB-Release: " + string(version) + "\r\n"
+        "User-Agent: ";
+    assert(!expected.empty());
+
+    // match expected against the start of the actual (stop before OS)
+    auto m = mismatch(expected.begin(), expected.end(), TestStream::m_requests.front().begin() );
+    if ( m.first != expected.end() )
+    {
+        cout << (int)*m.first << " != " << (int)*m.second << ", position=" << ( m.first - expected.begin() ) << endl;
+        REQUIRE_EQ( expected, TestStream::m_requests.front() );
+    }
+
+    REQUIRE_RC ( KClientHttpResultRelease ( rslt ) );
+}
+
+FIXTURE_TEST_CASE(HttpRequest_PUT_non_sra, HttpRequestFixture)
+{
+    MakeRequest( GetName() );
+
+    KClientHttpResult *rslt;
+    TestStream::AddResponse("HTTP/1.1 200 OK\r\n");
+    REQUIRE_RC ( KClientHttpRequestPUT ( m_req, & rslt, false ) );  // formas as non-SRA
+    REQUIRE_EQ( size_t(1), TestStream::m_requests.size() );
+
+    assert(m_req && m_req->http);
+    string expected("PUT ");
+    if (m_req->http->uf == eUFAbsolute)
+        expected += "http://HttpRequest_PUT_non_sra.com";
+    expected +=
+        "/blah HTTP/1.1\r\n"
+        "Host: HttpRequest_PUT_non_sra.com\r\n"
+        "\r\n";
+    assert(!expected.empty());
+
+    auto m = mismatch(expected.begin(), expected.end(), TestStream::m_requests.front().begin() );
+    if ( m.first != expected.end() )
+    {
+        cout << (int)*m.first << " != " << (int)*m.second << ", position=" << ( m.first - expected.begin() ) << endl;
+        REQUIRE_EQ( expected, TestStream::m_requests.front() );
+    }
+
+    REQUIRE_RC ( KClientHttpResultRelease ( rslt ) );
+}
+
 FIXTURE_TEST_CASE(HttpRequest_head_as_get, HttpRequestFixture)
 {
     MakeRequest( GetName() );
@@ -389,38 +458,6 @@ FIXTURE_TEST_CASE(HttpReliableRequest_POST_5xx_retry, HttpFixture)
     REQUIRE_RC ( KClientHttpResultRelease ( rslt ) );
 }
 
-/* VDB-3059: KHttpRequestPOST generates incorrect Content-Length after retry :
- it makes web server to return 400 Bad Request */
-FIXTURE_TEST_CASE(HttpReliableRequest_ContentLength, HttpFixture)
-{
-    /* calling good cgi returns 200 and resolved path */
-    REQUIRE_RC ( KNSManagerMakeReliableClientRequest ( m_mgr, &m_req, 0x01010000, NULL,
-        "https://trace.ncbi.nlm.nih.gov/Traces/names/names.fcgi" ) );
-    REQUIRE_RC ( KHttpRequestAddPostParam ( m_req, "acc=AAAB01" ) );
-    REQUIRE_RC ( KHttpRequestAddPostParam ( m_req, "accept-proto=https" ) );
-    REQUIRE_RC ( KHttpRequestAddPostParam ( m_req, "version=1.2" ) );
-
-    KHttpResult * rslt;
-    REQUIRE_RC ( KHttpRequestPOST ( m_req, & rslt ) );
-    uint32_t code = 0;
-    REQUIRE_RC ( KClientHttpResultStatus ( rslt, & code, NULL, 0, NULL ) );
-    REQUIRE_EQ ( code, 406u );
-
-    KStream * response;
-    REQUIRE_RC ( KHttpResultGetInputStream ( rslt, & response ) );
-    char buffer [ 512 ] = "";
-    size_t num_read = 0;
-    REQUIRE_RC (KStreamRead( response, buffer, sizeof buffer - 1,  &num_read ));
-    REQUIRE_LT ( num_read, sizeof buffer );
-    buffer [ num_read ] = '\0';
-    REQUIRE_EQ ( string ( buffer ), string ( 
-"This service has been deprecated, please update to the latest version of the toolkit. See https://github.com/ncbi/sra-tools/wiki/01.-Downloading-SRA-Toolkit\n"
-                                           ) );
-    REQUIRE_RC ( KStreamRelease ( response ) );
-
-    REQUIRE_RC ( KHttpResultRelease( rslt ) );
-}
-
 FIXTURE_TEST_CASE(HttpReliableRequest_BadCgi, HttpFixture)
 {
     /* calling non-existing cgi returns 404 */
