@@ -27,6 +27,11 @@
 
 #include <string>
 #include <cmath>
+#include <sstream>
+
+#include <kfc/except.h>
+
+#include <kfc/except.h>
 
 #include <vdb/database.h>
 #include <vdb/manager.h>
@@ -34,11 +39,14 @@
 #include <vdb/cursor.h>
 #include <vdb/table.h>
 
+#include "../libs/vdb/schema-priv.h"
 #include "../libs/schema/SchemaParser.hpp"
 #include "../libs/schema/ASTBuilder.hpp"
 
 #include <ktst/unit_test.hpp> // THROW_ON_RC
 
+const std :: string ScratchDir = "./db/";
+
 class WVDB_Fixture
 {
 public:
@@ -85,15 +93,19 @@ public:
         KDirectory* wd;
         KDirectoryNativeDir ( & wd );
         uint32_t mode = 0;
-        bool is_good = (0 == KDirectoryAccess_v1(wd, &mode, "%s", path.c_str()) & (mode & 0111) != 0);
+        bool is_good = (0 == KDirectoryAccess_v1(wd, &mode, "%s", path.c_str()) && (mode & 0111) != 0);
 
         KDirectoryRelease(wd);
         if (is_good) return;
         throw ncbi::NK::test_skipped("path not readable: " + path);
     }
 
-    void MakeDatabase ( const std :: string & p_schemaText, const std :: string & p_schemaSpec, const char * includes = nullptr )
+    void MakeDatabase ( const std :: string & p_testName,
+                        const std :: string & p_schemaText,
+                        const std :: string & p_schemaSpec,
+                        const char * includes = nullptr )
     {
+        m_databaseName = ScratchDir + p_testName;
         RemoveDatabase();
 
         THROW_ON_RC ( VDBManagerMakeUpdate ( & m_mgr, NULL ) );
@@ -151,12 +163,53 @@ public:
         THROW_ON_RC ( VCursorCommitRow ( p_cursor ) );
         THROW_ON_RC ( VCursorCloseRow ( p_cursor ) );
     }
+    void WriteRow ( VCursor * p_cursor, uint32_t p_colIdx, int64_t * p_value, uint32_t p_count )
+    {
+        THROW_ON_RC ( VCursorOpenRow ( p_cursor ) );
+        THROW_ON_RC ( VCursorWrite ( p_cursor, p_colIdx, sizeof(p_value)*8, p_value, 0, p_count ) );
+        THROW_ON_RC ( VCursorCommitRow ( p_cursor ) );
+        THROW_ON_RC ( VCursorCloseRow ( p_cursor ) );
+    }
 
     static std :: string ToCppString ( const String & p_str)
     {
         return std :: string ( p_str . addr, p_str . len );
     }
 
+    #include <sstream>
+    static std :: string DumpSchema(VSchema & p_schema, bool compact = true)
+    {
+
+        auto FlushSchema = [](void *fd, const void * buffer, size_t size) -> rc_t
+        {
+            std :: ostream & out = *static_cast < std :: ostream * > (fd);
+            out.write(static_cast < const char * > (buffer), size);
+            out.flush();
+            return 0;
+        };
+
+        bool free_intrinsic = false;
+        if ( p_schema . dad == nullptr )
+        {
+            THROW_ON_RC( VSchemaMakeIntrinsic ( (VSchema**) & p_schema . dad ) );
+            free_intrinsic = true;
+        }
+        std :: ostringstream out;
+        bool failed = VSchemaDump( & p_schema, compact ? sdmCompact : sdmPrint, 0, FlushSchema, &out);
+        if ( free_intrinsic )
+        {
+            VSchemaRelease( p_schema.dad );
+            p_schema.dad = nullptr;
+        }
+        if ( failed )
+        {
+            throw std :: runtime_error("DumpSchema failed");
+        }
+
+        return out.str();
+    }
+
+
     std :: string   m_databaseName;
     VDBManager *    m_mgr;
     VSchema *       m_schema;
@@ -180,8 +233,9 @@ public:
             return WVDB_Fixture :: ParseSchema ( p_schema, p_schemaText );
         }
 
+        HYBRID_FUNC_ENTRY( rcSRA, rcSchema, rcParsing );
         ncbi :: SchemaParser :: SchemaParser parser;
-        if ( ! parser . ParseString ( p_schemaText . c_str () ) )
+        if ( ! parser . ParseString ( ctx, p_schemaText . c_str () ) )
         {
             throw std :: logic_error ( std :: string ( "WVDB_Fixture::MakeDatabase : ParseString() failed: " ) + FormatErrorMessage ( * parser . GetErrors () . GetError ( 0 ) ) );
         }
@@ -190,8 +244,8 @@ public:
         {
             throw std :: logic_error ( "WVDB_Fixture::MakeDatabase : MoveParseTree() returned 0" );
         }
-        ncbi :: SchemaParser :: ASTBuilder builder ( p_schema );
-        ncbi :: SchemaParser :: AST * ast = builder . Build ( * parseTree, "", false );
+        ncbi :: SchemaParser :: ASTBuilder builder ( ctx, p_schema );
+        ncbi :: SchemaParser :: AST * ast = builder . Build ( ctx, * parseTree, "", false );
         if ( builder . GetErrorCount() != 0)
         {
             throw std :: logic_error ( std :: string ( "AST_Fixture::MakeAst : ASTBuilder::Build() failed: " ) + FormatErrorMessage ( * builder . GetErrors () . GetError ( 0 ) ) );
@@ -200,14 +254,20 @@ public:
         {
             throw std :: logic_error ( "AST_Fixture::MakeAst : ASTBuilder::Build() failed, no message!" );
         }
-        delete ast;
-        delete parseTree;
+        ncbi :: SchemaParser :: AST :: Destroy ( ast );
+        ncbi :: SchemaParser :: ParseTree :: Destroy ( parseTree );
+
+        if ( FAILED () )
+        {
+            throw std :: logic_error ( WHAT() );
+        }
     }
 
     static std :: string FormatErrorMessage( const ncbi :: SchemaParser :: ErrorReport :: Error & p_err )
     {
+        HYBRID_FUNC_ENTRY( rcSRA, rcSchema, rcParsing );
         char buf [1024];
-        if ( p_err . Format ( buf, sizeof ( buf ) ) )
+        if ( p_err . Format ( ctx, buf, sizeof ( buf ) ) )
         {
             return std :: string ( buf );
         }
