@@ -24,6 +24,11 @@
 
 #include "VDB_Fixture.hpp"
 
+#include <vdb/manager.h> // VDBManager
+#include <vdb/database.h>
+#include <vdb/table.h>
+#include <vdb/view.h>
+#include <vdb/cursor.h>
 #include <vdb/blob.h>
 #include <vdb/vdb-priv.h>
 #include <vdb/blob.h>
@@ -34,6 +39,7 @@ extern "C" {
 }
 
 #include <ktst/unit_test.hpp> // TEST_CASE
+#include <klib/container.h>
 #include <kfg/config.h>
 
 #include <sysalloc.h>
@@ -154,8 +160,8 @@ TEST_CASE( VdbMgr ) {
     mgr = NULL;
 
 }
-#if 0
-//#if _ARCH_BITS != 32
+
+#if CANT_WAIT_FOR_THIS_ONE_TO_FINISH_INVESTIGATE_AND_SPEEDUP
 TEST_CASE(SimultaneousCursors)
 {   // SRA-1669 WGS ALWZ01 cannot open multiple cursors with SEQUENCE.CONTIG_NAME column simultaneously (Win32)
 
@@ -175,7 +181,10 @@ TEST_CASE(SimultaneousCursors)
             REQUIRE_RC(VCursorPermitPostOpenAdd(cursor));
             REQUIRE_RC(VCursorOpen(cursor));
             uint32_t col_index;
+
+The following call takes an insane amount of time, abnout 3 minutes!
             REQUIRE_RC(VCursorAddColumn(cursor, &col_index, "CONTIG_NAME"));
+
         }
         for ( size_t i = 0; i < CURSOR_CNT; ++i ) {
             REQUIRE_RC(VCursorRelease(cursors[i]));
@@ -310,76 +319,6 @@ FIXTURE_TEST_CASE(VCursor_GetBlob_SRA, VDB_Fixture)
     }
 }
 
-FIXTURE_TEST_CASE(VCursor_GetBlob_WGS, VDB_Fixture)
-{   // single fragment per row, multiple rows per blob
-    static char const *columns[] = { "READ", 0 };
-    REQUIRE_RC ( Setup ( "ALWZ01", columns ) );
-
-    {
-        REQUIRE_RC ( VCursorSetRowId (curs, 1 ) );
-        REQUIRE_RC ( VCursorOpenRow (curs ) );
-
-        struct VBlob const *blob;
-        REQUIRE_RC ( VCursorGetBlob ( curs, &blob, col_idx[0] ) );
-
-        int64_t first;
-        uint64_t count;
-        REQUIRE_RC ( VBlobIdRange ( blob, &first, &count ) );
-        REQUIRE_EQ ( (int64_t)1, first );
-        REQUIRE_EQ ( (uint64_t)4, count );
-
-        REQUIRE_EQ ( (size_t)948, BlobBufferBytes ( blob ) );
-
-        //TODO: use row map to convert an offset into a rowId
-        REQUIRE_NOT_NULL ( blob->pm );
-        REQUIRE ( ! blob->pm->random_access );
-        REQUIRE_EQ ( (int)PageMap::eBlobPageMapOptimizedNone, (int)blob->pm->optimized );
-
-        REQUIRE_EQ ( (pm_size_t)4, blob->pm->leng_recs ); // not expanded ?
-        REQUIRE_NOT_NULL ( blob->pm->length );   // array of lengths, size blob->pm->leng_recs
-        REQUIRE_EQ ( (elem_count_t)227, blob->pm->length[0] );
-        REQUIRE_EQ ( (elem_count_t)245, blob->pm->length[1] );
-        REQUIRE_EQ ( (elem_count_t)211, blob->pm->length[2] );
-        REQUIRE_EQ ( (elem_count_t)265, blob->pm->length[3] );
-
-        REQUIRE_NOT_NULL ( blob->pm->leng_run ); // array of run lengths, size blob->pm->leng_recs
-        REQUIRE_EQ ( (row_count_t)1, blob->pm->leng_run[0] );
-        REQUIRE_EQ ( (row_count_t)1, blob->pm->leng_run[1] );
-        REQUIRE_EQ ( (row_count_t)1, blob->pm->leng_run[2] );
-        REQUIRE_EQ ( (row_count_t)1, blob->pm->leng_run[3] );
-        //NOTE: very unlikely to happen for READ column, but if leng_run > 1, the corresponding fragments should all appear in the search results
-
-        REQUIRE_EQ ( (pm_size_t)4, blob->pm->data_recs );
-
-        REQUIRE_EQ ( (row_count_t)4,  blob->pm->row_count );
-        REQUIRE_EQ ( (row_count_t)0,  blob->pm->pre_exp_row_count );
-
-        REQUIRE_RC ( VCursorCloseRow (curs ) );
-        REQUIRE_RC ( VBlobRelease ( (struct VBlob *) blob ) );
-    }
-
-    {
-        REQUIRE_RC ( VCursorSetRowId (curs, 5 ) );
-        REQUIRE_RC ( VCursorOpenRow (curs ) );
-
-        struct VBlob const *blob;
-
-        int64_t first;
-        uint64_t count;
-        REQUIRE_RC ( VCursorGetBlob ( curs, &blob, col_idx[0] ) );
-        REQUIRE_RC ( VBlobIdRange ( blob, &first, &count ) );
-        REQUIRE_EQ ( (int64_t)5, first );
-        REQUIRE_EQ ( (uint64_t)4, count );
-
-        REQUIRE_EQ ( (size_t)1184, BlobBufferBytes ( blob ) );
-
-        //TODO: use row map to convert an offset into a rowId
-
-        REQUIRE_RC ( VCursorCloseRow (curs ) );
-        REQUIRE_RC ( VBlobRelease ( (struct VBlob *) blob ) );
-    }
-}
-
 FIXTURE_TEST_CASE(VCursor_GetBlob_SequentialAccess, VDB_Fixture)
 {   // VDB-2858: sequential access to blobs broken
     static char const *columns[] = { "READ", 0 };
@@ -462,46 +401,6 @@ FIXTURE_TEST_CASE(VCursor_GetBlob_RandomAccess, VdbBlobFixture)
     REQUIRE ( CheckBlobRange ( 14001, 14001, 1 ) );
 }
 
-FIXTURE_TEST_CASE(PageMapIterator_WGS, VDB_Fixture)
-{   // single fragment per row, multiple rows per blob
-    static char const *columns[] = { "READ", 0 };
-    REQUIRE_RC ( Setup ( "ALWZ01", columns ) );
-
-    {
-        REQUIRE_RC ( VCursorSetRowId (curs, 1 ) );
-        REQUIRE_RC ( VCursorOpenRow (curs ) );
-
-        struct VBlob const *blob;
-        REQUIRE_RC ( VCursorGetBlob ( curs, &blob, col_idx[0] ) );
-
-        PageMapIterator pmIt;
-        REQUIRE_RC ( PageMapNewIterator ( (const PageMap*)blob->pm, &pmIt, 0, 4 ) );
-        REQUIRE_EQ ( (elem_count_t)227, PageMapIteratorDataLength ( &pmIt ) );
-        REQUIRE_EQ ( (elem_count_t)0, PageMapIteratorDataOffset ( &pmIt ) );
-        REQUIRE_EQ ( (row_count_t)1, PageMapIteratorRepeatCount ( &pmIt ) );
-
-        REQUIRE ( PageMapIteratorNext ( &pmIt ) );
-        REQUIRE_EQ ( (elem_count_t)245, PageMapIteratorDataLength ( &pmIt ) );
-        REQUIRE_EQ ( (elem_count_t)227, PageMapIteratorDataOffset ( &pmIt ) );
-        REQUIRE_EQ ( (row_count_t)1, PageMapIteratorRepeatCount ( &pmIt ) );
-
-        REQUIRE ( PageMapIteratorNext ( &pmIt ) );
-        REQUIRE_EQ ( (elem_count_t)211, PageMapIteratorDataLength ( &pmIt ) );
-        REQUIRE_EQ ( (elem_count_t)227+245, PageMapIteratorDataOffset ( &pmIt ) );
-        REQUIRE_EQ ( (row_count_t)1, PageMapIteratorRepeatCount ( &pmIt ) );
-
-        REQUIRE ( PageMapIteratorNext ( &pmIt ) );
-        REQUIRE_EQ ( (elem_count_t)265, PageMapIteratorDataLength ( &pmIt ) );
-        REQUIRE_EQ ( (elem_count_t)227+245+211, PageMapIteratorDataOffset ( &pmIt ) );
-        REQUIRE_EQ ( (row_count_t)1, PageMapIteratorRepeatCount ( &pmIt ) );
-
-        REQUIRE ( ! PageMapIteratorNext ( &pmIt ) );
-
-        REQUIRE_RC ( VCursorCloseRow (curs ) );
-        REQUIRE_RC ( VBlobRelease ( (struct VBlob *) blob ) );
-    }
-}
-
 FIXTURE_TEST_CASE ( VCursor_FindNextRowIdDirect, VDB_Fixture )
 {
     static char const *columns[] = { "SPOT_ID", "READ", 0 };
@@ -574,6 +473,77 @@ cout << out;
     }
 }
 #endif
+FIXTURE_TEST_CASE ( V2ParserError, VDB_Fixture )
+{   // This exercises an "extended schema" scenario, when a schema embedded
+    // in a table gets tacked onto the exsiting schema, possibly creating namespaces
+    // that obscure the parent schema (see callers of VSchemaParseTextCallback)
+    const VTable *tbl = NULL;
+    VSchema *schema = NULL;
+    REQUIRE_RC ( VDBManagerMakeSRASchema(mgr, &schema) );
+    REQUIRE_RC ( VDBManagerOpenTableRead ( mgr, &tbl, schema, "SRR053325" ) );
+    REQUIRE_RC ( VTableCreateCursorRead(tbl, &curs) );
+    uint32_t colIdx;
+    REQUIRE_RC ( VCursorAddColumn ( curs, & colIdx, "READ" ) );
+    REQUIRE_RC ( VCursorOpen (curs ) );
+    REQUIRE_RC ( VSchemaRelease ( schema ) );
+    REQUIRE_RC ( VTableRelease ( tbl ) );
+}
+
+rc_t CC FlushSchema(void *fd, const void * buffer, size_t size)
+{
+    ostream & out = *static_cast < ostream * > (fd);
+    out.write(static_cast < const char * > (buffer), size);
+    out.flush();
+    return 0;
+}
+
+#include <sstream>
+string
+DumpSchema(const VSchema * p_schema)
+{
+    ostringstream out;
+    if (VSchemaDump(p_schema, sdmPrint, 0, FlushSchema, &out) != 0)
+    {
+        throw runtime_error("DumpSchema failed");
+    }
+    return out.str();
+}
+
+TEST_CASE(View_On_An_Existing_Schema)
+{
+    const VDBManager * mgr;
+    REQUIRE_RC(VDBManagerMakeRead(&mgr, NULL));
+
+    const VDatabase *db = NULL;
+    const char * acc = "SRR1063272";
+
+    const string schemaText =
+        "version 2; "
+        "view V1#1 < NCBI:align:tbl:align_cmn align >"
+        "{"
+        "    column I64 SEQ_SPOT_ID = align.SEQ_SPOT_ID;"
+        "}";
+
+    VSchema * schema;
+    REQUIRE_RC(VDBManagerOpenDBRead(mgr, &db, NULL, acc));
+    REQUIRE_RC(VDatabaseOpenSchema(db, (const VSchema **)& schema));
+    REQUIRE_RC(VSchemaParseText(schema, "", schemaText.c_str(), schemaText.size()));
+
+    const VView * view;
+    REQUIRE_RC(VDBManagerOpenView(mgr, &view, schema, "V1"));
+
+    const VTable * tbl_pa;
+    REQUIRE_RC(VDatabaseOpenTableRead(db, &tbl_pa, "PRIMARY_ALIGNMENT")); // this used to mess up VSchema
+
+    string d = DumpSchema(schema);
+
+    REQUIRE_RC(VTableRelease(tbl_pa));
+    REQUIRE_RC(VViewRelease(view));
+    REQUIRE_RC(VSchemaRelease(schema));
+
+    REQUIRE_RC(VDatabaseRelease(db));
+    REQUIRE_RC(VDBManagerRelease(mgr));
+}
 
 //////////////////////////////////////////// Main
 extern "C"
